#!/usr/bin/env python3
# Copyright 2016 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Definition of targets to build distribution packages."""

import os.path
import sys

sys.path.insert(0, os.path.abspath(".."))
import python_utils.jobset as jobset

REPORT_BASE_PATH = os.getenv("GRPC_TEST_REPORT_BASE_DIR", os.path.abspath("."))


def create_docker_jobspec(
    name,
    dockerfile_dir,
    shell_command,
    environ={},
    flake_retries=0,
    timeout_retries=0,
):
    """Creates jobspec for a task running under docker."""
    environ = environ.copy()

    docker_args = []
    for k, v in list(environ.items()):
        docker_args += ["-e", "%s=%s" % (k, v)]
    docker_env = {
        "DOCKERFILE_DIR": dockerfile_dir,
        "DOCKER_RUN_SCRIPT": "tools/run_tests/dockerize/docker_run.sh",
        "DOCKER_RUN_SCRIPT_COMMAND": shell_command,
        "OUTPUT_DIR": "artifacts",
    }
    jobspec = jobset.JobSpec(
        cmdline=["tools/run_tests/dockerize/build_and_run_docker.sh"]
        + docker_args,
        environ=docker_env,
        shortname="build_package.%s" % (name),
        timeout_seconds=30 * 60,
        flake_retries=flake_retries,
        timeout_retries=timeout_retries,
        logfilename=os.path.abspath(
            f"{REPORT_BASE_PATH}/reports/package.{name}.log"
        ),
    )
    return jobspec


def create_jobspec(
    name,
    cmdline,
    environ=None,
    cwd=None,
    shell=False,
    flake_retries=0,
    timeout_retries=0,
    cpu_cost=1.0,
):
    """Creates jobspec."""
    jobspec = jobset.JobSpec(
        cmdline=cmdline,
        environ=environ,
        cwd=cwd,
        shortname="build_package.%s" % (name),
        timeout_seconds=10 * 60,
        flake_retries=flake_retries,
        timeout_retries=timeout_retries,
        cpu_cost=cpu_cost,
        shell=shell,
        logfilename=os.path.abspath(
            f"{REPORT_BASE_PATH}/reports/package.{name}.log"
        ),
    )
    return jobspec


class CSharpPackage:
    """Builds C# packages."""

    def __init__(self, platform):
        self.platform = platform
        self.labels = ["package", "csharp", self.platform]
        self.name = "csharp_package_nuget_%s" % self.platform
        self.labels += ["nuget"]

    def pre_build_jobspecs(self):
        return []

    def build_jobspec(self, inner_jobs=None):
        del inner_jobs  # arg unused as there is little opportunity for parallelizing
        environ = {
            "GRPC_CSHARP_BUILD_SINGLE_PLATFORM_NUGET": os.getenv(
                "GRPC_CSHARP_BUILD_SINGLE_PLATFORM_NUGET", ""
            )
        }

        build_script = "src/csharp/build_nuget.sh"

        if self.platform == "linux":
            return create_docker_jobspec(
                self.name,
                "tools/dockerfile/test/csharp_debian11_x64",
                build_script,
                environ=environ,
            )
        else:
            repo_root = os.path.join(
                os.path.dirname(os.path.abspath(__file__)), "..", "..", ".."
            )
            environ["EXTERNAL_GIT_ROOT"] = repo_root
            return create_jobspec(
                self.name, ["bash", build_script], environ=environ
            )

    def __str__(self):
        return self.name


class RubyPackage:
    """Collects ruby gems created in the artifact phase"""

    def __init__(self):
        self.name = "ruby_package"
        self.labels = ["package", "ruby", "linux"]

    def pre_build_jobspecs(self):
        return []

    def build_jobspec(self, inner_jobs=None):
        del inner_jobs  # arg unused as this step simply collects preexisting artifacts
        return create_docker_jobspec(
            self.name,
            "tools/dockerfile/grpc_artifact_manylinux2014_x64",
            "tools/run_tests/artifacts/build_package_ruby.sh",
        )


class PythonPackage:
    """Collects python eggs and wheels created in the artifact phase"""

    def __init__(self, platform="", arch=""):
        self.name = "python_package"
        self.labels = ["package", "python", "linux"]
        self.platform = platform
        self.arch = arch
        if self.platform:
            self.labels.append(platform)
            self.name += "_" + platform
        if self.arch:
            self.labels.append(arch)
            self.name += "_" + arch

            if self.platform == "any" and arch == "aarch64":
                self.labels.append("exclude_in_collect_all_packages")

    def pre_build_jobspecs(self):
        return []

    def build_jobspec(self, inner_jobs=None):
        del inner_jobs  # arg unused as this step simply collects preexisting artifacts
        # since the python package build does very little, we can use virtually
        # any image that has new-enough python, so reusing one of the images used
        # for artifact building seems natural.
        dockerfile_dir = (
            "tools/dockerfile/grpc_artifact_python_manylinux2014_x64"
        )
        shell_command = "tools/run_tests/artifacts/package_python.sh"
        environ = {
            "PYTHON": "/opt/python/cp39-cp39/bin/python",
            "ARTIFACT_PREFIX": "python_",
            "EXCLUDE_PATTERNS": "python_musllinux_1_2_aarch64_* python_manylinux2014_aarch64_*",
        }

        if self.platform == "any":
            # all the artifact builder configurations generate an equivalent
            # grpcio-VERSION.tar.gz source distribution package and
            # grpcio-VERSION-py3-none-any.whl file. Only one of them is needed
            # in the artifacts/ directory. Hence copy it separately exactly once
            # using a separate package target
            # Note: Source installation using *tar.gz file is not tested for
            # aarch64 jobs, so these files will not be copied in the
            # 'Distribution Tests Arm64' job
            shell_command = "tools/run_tests/artifacts/package_python_noarch.sh"
            if self.arch == "aarch64":
                dockerfile_dir = "tools/dockerfile/grpc_artifact_python_manylinux2014_aarch64"
                environ["ARTIFACT_PREFIX"] = "python_manylinux2014_aarch64_"
            else:
                # noarch files in all platform-arch combinations are going to be
                # the same, so specify any one combination as prefix
                environ["ARTIFACT_PREFIX"] = "python_manylinux2014_x64_"

        elif self.arch == "aarch64":
            if "musllinux_1_2" in self.platform:
                dockerfile_dir = "tools/dockerfile/grpc_artifact_python_musllinux_1_2_aarch64"
                environ["ARTIFACT_PREFIX"] = "python_musllinux_1_2_aarch64_"

            elif "manylinux2014" in self.platform:
                dockerfile_dir = "tools/dockerfile/grpc_artifact_python_manylinux2014_aarch64"
                environ["ARTIFACT_PREFIX"] = "python_manylinux2014_aarch64_"
            environ["EXCLUDE_PATTERNS"] = ""

        return create_docker_jobspec(
            self.name,
            dockerfile_dir,
            shell_command,
            environ=environ,
        )


class PHPPackage:
    """Copy PHP PECL package artifact"""

    def __init__(self):
        self.name = "php_package"
        self.labels = ["package", "php", "linux"]

    def pre_build_jobspecs(self):
        return []

    def build_jobspec(self, inner_jobs=None):
        del inner_jobs  # arg unused as this step simply collects preexisting artifacts
        return create_docker_jobspec(
            self.name,
            "tools/dockerfile/grpc_artifact_manylinux2014_x64",
            "tools/run_tests/artifacts/build_package_php.sh",
        )


def targets():
    """Gets list of supported targets"""
    return [
        CSharpPackage("linux"),
        CSharpPackage("macos"),
        CSharpPackage("windows"),
        RubyPackage(),
        PythonPackage(),
        PythonPackage("any"),
        PythonPackage("musllinux_1_2", "aarch64"),
        PythonPackage("manylinux2014", "aarch64"),
        PythonPackage("any", "aarch64"),
        PHPPackage(),
    ]



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/extensions/filters/http/mcp_router/v3/mcp_router.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/extensions/filters/http/mcp_router/v3/mcp_router.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from xds.annotations.v3 import status_pb2 as xds_dot_annotations_dot_v3_dot_status__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n<envoy/extensions/filters/http/mcp_router/v3/mcp_router.proto\x12+envoy.extensions.filters.http.mcp_router.v3\x1a\x1egoogle/protobuf/duration.proto\x1a\x1fxds/annotations/v3/status.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\xd3\x02\n\tMcpRouter\x12R\n\x07servers\x18\x01 \x03(\x0b\x32\x41.envoy.extensions.filters.http.mcp_router.v3.McpRouter.McpBackend\x1ar\n\nMcpBackend\x12\x0c\n\x04name\x18\x01 \x01(\t\x12V\n\x0bmcp_cluster\x18\x02 \x01(\x0b\x32\x41.envoy.extensions.filters.http.mcp_router.v3.McpRouter.McpCluster\x1a~\n\nMcpCluster\x12\x18\n\x07\x63luster\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\x12\x0c\n\x04path\x18\x02 \x01(\t\x12*\n\x07timeout\x18\x03 \x01(\x0b\x32\x19.google.protobuf.Duration\x12\x1c\n\x14host_rewrite_literal\x18\x04 \x01(\tB\xbe\x01\n9io.envoyproxy.envoy.extensions.filters.http.mcp_router.v3B\x0eMcpRouterProtoP\x01Z_github.com/envoyproxy/go-control-plane/envoy/extensions/filters/http/mcp_router/v3;mcp_routerv3\xba\x80\xc8\xd1\x06\x02\x10\x02\xd2\xc6\xa4\xe1\x06\x02\x08\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.extensions.filters.http.mcp_router.v3.mcp_router_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n9io.envoyproxy.envoy.extensions.filters.http.mcp_router.v3B\016McpRouterProtoP\001Z_github.com/envoyproxy/go-control-plane/envoy/extensions/filters/http/mcp_router/v3;mcp_routerv3\272\200\310\321\006\002\020\002\322\306\244\341\006\002\010\001'
  _globals['_MCPROUTER_MCPCLUSTER'].fields_by_name['cluster']._loaded_options = None
  _globals['_MCPROUTER_MCPCLUSTER'].fields_by_name['cluster']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_MCPROUTER']._serialized_start=231
  _globals['_MCPROUTER']._serialized_end=570
  _globals['_MCPROUTER_MCPBACKEND']._serialized_start=328
  _globals['_MCPROUTER_MCPBACKEND']._serialized_end=442
  _globals['_MCPROUTER_MCPCLUSTER']._serialized_start=444
  _globals['_MCPROUTER_MCPCLUSTER']._serialized_end=570
# @@protoc_insertion_point(module_scope)



# Copyright 2017 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests that a channel will reconnect if a connection is dropped"""
from __future__ import annotations

from collections.abc import Sequence
import contextlib
import logging
import socket
import time
from typing import Callable, Final, Optional, Tuple
import unittest

import grpc
from grpc import _channel as grpc_channel_internal
from grpc import _common as grpc_common_internal
from grpc import _typing as grpc_typing
from grpc.framework.foundation import logging_pool

from tests.unit.framework import common as test_common
from tests.unit.framework.common import test_constants

# Constants.
# Using a relatively low value compared to the default:
# we're expecting RPCs to deadline when the server is down.
_RPC_TIMEOUT = test_constants.SHORT_TIMEOUT / 10.0

# Type aliases.
# Set type to TypeAlias when typing_extensions installed.
_CheckResultFnType = Callable[[Optional[str], Optional[grpc.RpcError]], bool]
_CallRpcReturnType = Tuple[Optional[str], Optional[grpc.RpcError]]


class TestServer:
    SERVICE_NAME: Final[str] = "test"
    METHOD_NAME: Final[str] = "EchoServerId"
    _SERVER_OPTS: Final[Sequence[grpc_typing.ChannelArgumentType]] = (
        ("grpc.so_reuseport", 1),
    )

    pool: logging_pool._LoggingPool
    host: str
    port: int
    _sock: socket.socket

    server_id: int = 0
    server: Optional[grpc.Server] = None

    def __init__(self):
        self.pool = logging_pool.pool(test_constants.THREAD_CONCURRENCY)
        self.host, self.port, self._sock = test_common.get_socket(listen=False)

    @property
    def addr(self) -> str:
        return f"{self.host}:{self.port}"

    @property
    def service(self) -> str:
        return grpc_common_internal.fully_qualified_method(
            self.SERVICE_NAME, self.METHOD_NAME
        )

    def serve(self) -> None:
        if self.server:
            raise ValueError("Already serving")

        self.server_id += 1

        server: grpc.Server = grpc.server(self.pool, options=self._SERVER_OPTS)
        self.server = server
        server.add_registered_method_handlers(
            self.SERVICE_NAME, self._make_handlers()
        )
        server.add_insecure_port(self.addr)

        logging.info("[SERVER #%s] Starting on %s", self.server_id, self.addr)
        server.start()

    def terminate(self) -> None:
        if not self.server:
            raise ValueError("Nothing to terminate, not serving")
        logging.info("[SERVER #%s] Stopping the server", self.server_id)
        self.server.stop(grace=None)
        self.server = None

    def shutdown_now(self) -> None:
        logging.debug("[SERVER #%s] Shutdown all", self.server_id)
        cleanup = contextlib.ExitStack()
        cleanup.callback(self._sock.close)
        if self.server:
            cleanup.callback(self.server.stop, grace=None)
        cleanup.callback(self.pool.shutdown, wait=False)
        cleanup.close()
        logging.debug("[SERVER #%s] Shutdown complete", self.server_id)

    def _make_handlers(self) -> dict[str, grpc.RpcMethodHandler]:
        return {
            self.METHOD_NAME: grpc.unary_unary_rpc_method_handler(
                self._handle_unary_unary
            )
        }

    def _handle_unary_unary(
        self,
        request: bytes,
        unused_servicer_context,
    ) -> bytes:
        req: str = request.decode()
        logging.info("[SERVER #%s] Received request=%r", self.server_id, req)
        return f"{req}-server#{self.server_id}".encode()


class ReconnectTest(unittest.TestCase):
    test_server: Optional[TestServer] = None
    channel: Optional[grpc.Channel] = None
    req_id: int = 0

    def setUp(self) -> None:
        self.test_server = TestServer()

    def tearDown(self) -> None:
        try:
            if self.channel:
                self.channel.close()
        finally:
            if self.test_server:
                self.test_server.shutdown_now()

    def test_reconnect(self) -> None:
        if not self.test_server:
            raise AssertionError("TestServer not initialized.")

        # Start the server.
        self.test_server.serve()

        # Start the client.
        channel: grpc.Channel = grpc.insecure_channel(self.test_server.addr)
        self.channel = channel
        rpc_callable: grpc.UnaryUnaryMultiCallable = channel.unary_unary(
            self.test_server.service,
            _registered_method=True,  # to be close to generated stubs.
        )

        # Step 1. Initial verification: assert good calls.
        num_consequent_successes = 3
        responses, ok = self._call_until(
            rpc_callable,
            attempts=num_consequent_successes * 10,
            min_consequent_checks=num_consequent_successes,
            check_result_fn=lambda _, err: not err,
            rpc_timeout=_RPC_TIMEOUT,
        )
        if not ok:
            self.fail(
                "[TEST] Initial verification failed: didn't receive "
                f"{num_consequent_successes} good calls in sequence"
            )
        self.assertEqual(
            responses[-1][0],
            f"req{self.req_id}-server#1",
            "Expected initial responses to be served by server#1",
        )
        logging.info(
            "[TEST] Initial verification success: detected %s good calls",
            num_consequent_successes,
        )

        # Step 2. Shutdown verification: assert failed calls.
        self.test_server.terminate()

        # Important: we do not set the timeout to force transport.
        # Without the timeout, the deadline may arrive sooner than the client
        # channel transitions CONNECTING -> TRANSIENT_FAILURE.
        num_consequent_failures = 3
        responses, ok = self._call_until(
            rpc_callable,
            attempts=num_consequent_failures * 10,
            min_consequent_checks=num_consequent_failures,
            check_result_fn=lambda resp, _: not resp,
        )
        if not ok:
            self.fail(
                "[TEST] Shutdown verification failed: didn't receive "
                f"{num_consequent_failures} failed calls in sequence"
            )
        _, err = responses[-1]
        logging.info(
            "[TEST] Shutdown verification success: detected %s failed calls",
            num_consequent_successes,
        )

        # Step 3. Recovery verification: assert some number of bad calls,
        # followed by good calls again.
        self.test_server.serve()

        # By default, the channel connectivity is checked every 5s
        # GRPC_CLIENT_CHANNEL_BACKUP_POLL_INTERVAL_MS can be set to change
        # this, but we want to be close to the actual library use.
        #
        # To limit testing time in the negative case, we set,
        num_consequent_successes: int = 3
        expected_recovery_after_sec: Final[float] = 5.1
        sleep_sec_check_fail: Final[float] = 0.5
        failed_call_approx_sec = sleep_sec_check_fail + _RPC_TIMEOUT
        # The number of expected retries needed to recover plus number
        # of expected successes. Multiply by two for some headroom.
        attempts: int = 2 * (
            int(expected_recovery_after_sec / failed_call_approx_sec)
            + num_consequent_successes
        )

        logging.info("[TEST] Waiting for recovery, max_attempts: %r", attempts)
        responses, ok = self._call_until(
            rpc_callable,
            attempts=attempts,
            sleep_sec_check_fail=sleep_sec_check_fail,
            min_consequent_checks=num_consequent_successes,
            check_result_fn=lambda _, err: not err,
            rpc_timeout=_RPC_TIMEOUT,
        )

        num_failed_calls = 0
        for resp, _ in responses:
            if resp:
                break
            num_failed_calls += 1

        if not ok:
            self.fail(
                "[TEST] Recovery verification failed: didn't receive "
                f"{num_consequent_successes} good calls in sequence. "
                f"Failed calls: {num_failed_calls} out of {attempts}"
            )
        self.assertEqual(
            responses[-1][0],
            f"req{self.req_id}-server#2",
            "Recovery verification: expected responses to be served by server#2",
        )

        logging.info(
            "[TEST] Recovery verification success: "
            "detected %s failed calls, followed by %s good calls",
            num_failed_calls,
            num_consequent_successes,
        )

    def _call_rpc(
        self,
        rpc_callable: grpc.UnaryUnaryMultiCallable,
        timeout: Optional[float] = None,
    ) -> _CallRpcReturnType:
        # Prep request.
        self.req_id += 1
        req = f"req{self.req_id}"
        logging.info("[CLIENT] Sending request=%s", req)
        try:
            response: bytes = rpc_callable(req.encode(), timeout=timeout)
            resp: str = response.decode()
            logging.info("[CLIENT] Received response=%r", resp)
            return resp, None
        except grpc.RpcError as err:
            logging.info(
                "[CLIENT] Failed request=%s: %s",
                req,
                self._pretty_print_rpc_error(err),
            )
            return None, err

    @classmethod
    def _pretty_print_rpc_error(cls, err: grpc.RpcError) -> str:
        if hasattr(grpc_channel_internal, "_InactiveRpcError") and isinstance(
            err, grpc_channel_internal._InactiveRpcError
        ):
            code = err.code()
            code_name = code.name if code is not None else "(unknown)"
            return f"code={code_name}, error={err.debug_error_string()}"

        return f"{err!r}"

    def _call_until(
        self,
        rpc_callable: grpc.UnaryUnaryMultiCallable,
        *,
        check_result_fn: _CheckResultFnType,
        attempts: int,
        min_consequent_checks: int = 1,
        sleep_sec_check_pass: float = 0,
        sleep_sec_check_fail: float = 0,
        rpc_timeout: Optional[float] = None,
    ) -> Tuple[list[_CallRpcReturnType], bool]:
        responses = []
        successes = 0
        for attempt in range(attempts):
            sleep_sec = 0
            result = self._call_rpc(rpc_callable, rpc_timeout)
            responses.append(result)

            # Check if success and early exit.
            if check_result_fn(*result):
                successes += 1
                sleep_sec = sleep_sec_check_pass
                if successes >= min_consequent_checks:
                    return responses, True
            else:
                sleep_sec = sleep_sec_check_fail

            # Continue after sleep (except the last attempt).
            if sleep_sec and attempt < attempts - 1:
                logging.debug("[TEST] Sleeping for %ss", sleep_sec)
                time.sleep(sleep_sec)

        return responses, False


if __name__ == "__main__":
    # absl-style logging.
    logging.basicConfig(
        level=logging.INFO,
        style="{",
        format=(
            "{levelname[0]}{asctime}.{msecs:03.0f} {thread} "
            "{filename}:{lineno}] {message}"
        ),
        datefmt="%m%d %H:%M:%S",
    )
    unittest.main()



# Copyright 2021 The gRPC Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Test the time_remaining() method of async ServicerContext."""

import asyncio
import datetime
import logging
import unittest

import grpc
from grpc import aio

from tests_aio.unit._common import ADHOC_METHOD
from tests_aio.unit._common import AdhocGenericHandler
from tests_aio.unit._test_base import AioTestBase

_REQUEST = b"\x09\x05"
_REQUEST_TIMEOUT_S = datetime.timedelta(seconds=5).total_seconds()


class TestServerTimeRemaining(AioTestBase):
    async def setUp(self):
        # Create async server
        self._server = aio.server(options=(("grpc.so_reuseport", 0),))
        self._adhoc_handlers = AdhocGenericHandler()
        self._server.add_generic_rpc_handlers((self._adhoc_handlers,))
        port = self._server.add_insecure_port("[::]:0")
        address = "localhost:%d" % port
        await self._server.start()
        # Create async channel
        self._channel = aio.insecure_channel(address)

    async def tearDown(self):
        await self._channel.close()
        await self._server.stop(None)

    async def test_servicer_context_time_remaining(self):
        seen_time_remaining = []

        @grpc.unary_unary_rpc_method_handler
        def log_time_remaining(
            request: bytes, context: grpc.ServicerContext
        ) -> bytes:
            seen_time_remaining.append(context.time_remaining())
            return b""

        # Check if the deadline propagates properly
        self._adhoc_handlers.set_adhoc_handler(log_time_remaining)
        await self._channel.unary_unary(ADHOC_METHOD)(
            _REQUEST, timeout=_REQUEST_TIMEOUT_S
        )
        self.assertGreater(seen_time_remaining[0], _REQUEST_TIMEOUT_S / 2)
        # Check if there is no timeout, the time_remaining will be None
        self._adhoc_handlers.set_adhoc_handler(log_time_remaining)
        await self._channel.unary_unary(ADHOC_METHOD)(_REQUEST)
        self.assertIsNone(seen_time_remaining[1])


if __name__ == "__main__":
    logging.basicConfig(level=logging.DEBUG)
    unittest.main(verbosity=2)



# Copyright 2017 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


cdef class _Tag:

  cdef BaseEvent event(self, grpc_event c_event):
    raise NotImplementedError()


cdef class _ConnectivityTag(_Tag):

  def __cinit__(self, user_tag):
    self._user_tag = user_tag

  cdef ConnectivityEvent event(self, grpc_event c_event):
    return ConnectivityEvent(c_event.type, c_event.success, self._user_tag)


cdef class _RequestCallTag(_Tag):

  def __cinit__(self, user_tag):
    self._user_tag = user_tag
    self.call = None
    self.call_details = None

  cdef void prepare(self) except *:
    self.call = Call()
    self.call_details = CallDetails()
    grpc_metadata_array_init(&self.c_invocation_metadata)

  cdef RequestCallEvent event(self, grpc_event c_event):
    cdef tuple invocation_metadata = _metadata(&self.c_invocation_metadata)
    grpc_metadata_array_destroy(&self.c_invocation_metadata)
    return RequestCallEvent(
        c_event.type, c_event.success, self._user_tag, self.call,
        self.call_details, invocation_metadata)


cdef class _BatchOperationTag:

  def __cinit__(self, user_tag, operations, call):
    self._user_tag = user_tag
    self._operations = operations
    self._retained_call = call

  cdef void prepare(self) except *:
    cdef Operation operation
    self.c_nops = 0 if self._operations is None else len(self._operations)
    if 0 < self.c_nops:
      self.c_ops = <grpc_op *>gpr_malloc(sizeof(grpc_op) * self.c_nops)
      for index, operation in enumerate(self._operations):
        operation.c()
        self.c_ops[index] = operation.c_op

  cdef BatchOperationEvent event(self, grpc_event c_event):
    cdef Operation operation
    if 0 < self.c_nops:
      for operation in self._operations:
        operation.un_c()
      gpr_free(self.c_ops)
      return BatchOperationEvent(
          c_event.type, c_event.success, self._user_tag, self._operations)
    else:
      return BatchOperationEvent(
          c_event.type, c_event.success, self._user_tag, ())


cdef class _ServerShutdownTag(_Tag):

  def __cinit__(self, user_tag, shutting_down_server):
    self._user_tag = user_tag
    self._shutting_down_server = shutting_down_server

  cdef ServerShutdownEvent event(self, grpc_event c_event):
    self._shutting_down_server.notify_shutdown_complete()
    return ServerShutdownEvent(c_event.type, c_event.success, self._user_tag)



# Copyright 2015 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""The Python implementation of the GRPC interoperability test server."""

from concurrent import futures
import logging

from absl import app
from absl.flags import argparse_flags
import grpc

from src.proto.grpc.testing import test_pb2_grpc
from tests.interop import resources
from tests.interop import service
from tests.unit import test_common

logging.basicConfig()
_LOGGER = logging.getLogger(__name__)


def parse_interop_server_arguments(argv):
    parser = argparse_flags.ArgumentParser()
    parser.add_argument(
        "--port", type=int, required=True, help="the port on which to serve"
    )
    parser.add_argument(
        "--use_tls",
        default=False,
        type=resources.parse_bool,
        help="require a secure connection",
    )
    parser.add_argument(
        "--use_alts",
        default=False,
        type=resources.parse_bool,
        help="require an ALTS connection",
    )
    return parser.parse_args(argv[1:])


def get_server_credentials(use_tls):
    if use_tls:
        private_key = resources.private_key()
        certificate_chain = resources.certificate_chain()
        return grpc.ssl_server_credentials(((private_key, certificate_chain),))
    else:
        return grpc.alts_server_credentials()


def serve(args):
    server = test_common.test_server()
    test_pb2_grpc.add_TestServiceServicer_to_server(
        service.TestService(), server
    )
    if args.use_tls or args.use_alts:
        credentials = get_server_credentials(args.use_tls)
        server.add_secure_port("[::]:{}".format(args.port), credentials)
    else:
        server.add_insecure_port("[::]:{}".format(args.port))

    server.start()
    _LOGGER.info("Server serving.")
    server.wait_for_termination()
    _LOGGER.info("Server stopped; exiting.")


if __name__ == "__main__":
    app.run(serve, flags_parser=parse_interop_server_arguments)



#!/usr/bin/env python3

# Copyright 2021 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import re
import sys

os.chdir(os.path.join(os.path.dirname(sys.argv[0]), "../../.."))

BAD_REGEXES = [
    (r'\n#include "include/(.*)"', r"\n#include <\1>"),
    (r'\n#include "grpc(.*)"', r"\n#include <grpc\1>"),
    (r"\n#include <gtest(.*)>", r'\n#include "gtest\1"'),
    (r"\n#include <gmock(.*)>", r'\n#include "gmock\1"'),
]

fix = sys.argv[1:] == ["--fix"]
if fix:
    print("FIXING!")


def check_include_style(directory_root):
    bad_files = []
    for root, dirs, files in os.walk(directory_root):
        for filename in files:
            path = os.path.join(root, filename)
            if os.path.splitext(path)[1] not in [".c", ".cc", ".h"]:
                continue
            if filename.endswith(".pb.h") or filename.endswith(".pb.c"):
                continue
            # Skip check for upb generated code.
            if (
                filename.endswith(".upb.h")
                or filename.endswith(".upbdefs.h")
                or filename.endswith(".upbdefs.c")
            ):
                continue
            with open(path) as f:
                text = f.read()
            original = text
            for regex, replace in BAD_REGEXES:
                text = re.sub(regex, replace, text)
            if text != original:
                bad_lines = []
                for line_number, (original_line, text_line) in enumerate(
                    zip(original.splitlines(), text.splitlines())
                ):
                    if original_line != text_line:
                        bad_lines.append(
                            (line_number + 1, original_line, text_line)
                        )
                bad_files.append((path, bad_lines))
                if fix:
                    with open(path, "w") as f:
                        f.write(text)
    return bad_files


all_bad_files = []
all_bad_files += check_include_style(os.path.join("src", "core"))
all_bad_files += check_include_style(os.path.join("src", "cpp"))
all_bad_files += check_include_style(os.path.join("test", "core"))
all_bad_files += check_include_style(os.path.join("test", "cpp"))
all_bad_files += check_include_style(os.path.join("include", "grpc"))
all_bad_files += check_include_style(os.path.join("include", "grpcpp"))

if all_bad_files:
    for f, bad_lines in all_bad_files:
        print("%s has badly formed grpc system header files:" % f)
        for line_number, original_line, text_line in bad_lines:
            print(
                "\tline %d: %s -> %s" % (line_number, original_line, text_line)
            )
    sys.exit(1)



# Copyright 2019 The gRPC Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests for an actual dns resolution."""

import logging
import unittest

import grpc

from tests.unit import test_common
from tests.unit.framework.common import test_constants

_SERVICE_NAME = "test"
_METHOD = "METHOD"
_REQUEST = b"\x00\x00\x00"
_RESPONSE = _REQUEST


_METHOD_HANDLERS = {
    _METHOD: grpc.unary_unary_rpc_method_handler(
        lambda request, unused_context: request,
    )
}


class DNSResolverTest(unittest.TestCase):
    def setUp(self):
        self._server = test_common.test_server()
        self._server.add_registered_method_handlers(
            _SERVICE_NAME, _METHOD_HANDLERS
        )
        self._port = self._server.add_insecure_port("[::]:0")
        self._server.start()

    def tearDown(self):
        self._server.stop(None)

    def test_connect_loopback(self):
        # NOTE(https://github.com/grpc/grpc/issues/18422)
        # In short, Gevent + C-Ares = Segfault. The C-Ares driver is not
        # supported by custom io manager like "gevent"
        # NOTE(b/201064791): use loopback46.unittest.grpc.io since
        # it returns the expected responses even when DNS64 dns servers
        # are used on the test worker (and for purposes of this
        # test the use of loopback4 vs loopback46 makes no difference).
        with grpc.insecure_channel(
            "loopback46.unittest.grpc.io:%d" % self._port
        ) as channel:
            self.assertEqual(
                channel.unary_unary(
                    grpc._common.fully_qualified_method(_SERVICE_NAME, _METHOD),
                    _registered_method=True,
                )(
                    _REQUEST,
                    timeout=10,
                ),
                _RESPONSE,
            )


if __name__ == "__main__":
    logging.basicConfig()
    unittest.main(verbosity=2)



# Copyright 2022 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

QUERY = """
#standardSQL

-- See https://console.cloud.google.com/bigquery?sq=830293263384:5a8549832dfb48d9b2c04312a4ae3181 for the original query

WITH

  runs AS (
  SELECT
    RTRIM(LTRIM(REGEXP_REPLACE(test_target, r'(@poller=.+)', ''))) AS test_binary,
    REGEXP_EXTRACT(test_target, r'poller=(\w+)') AS poll_strategy,
    job_name,
    test_target,
    test_class_name,
    CASE
      # in case of timeout / retry / segfault the "test_case" fields will contain weird stuff
      # e.g. "test_shard2_run0_attempt0" or "shard_2/20"
      # when aggregating, we want to display all of these as a single category of problems
      WHEN test_case like 'test_shard_%_run%_attempt%' THEN 'CANNOT_DETERMINE'
      WHEN test_case like '%shard_%/%' THEN 'CANNOT_DETERMINE'
      # when test_case looks suspiciously like test_target
      # its value is probably meaningless and it means that the entire target has failed
      # e.g. test_target="//test/cpp/client:destroy_grpclb_channel_with_active_connect_stress_test" and test_case="test/cpp/client/destroy_grpclb_channel_with_active_connect_stress_test.exe"                         
      WHEN STRPOS(test_case, REPLACE(SUBSTR(test_target, 3), ":", "/")) != 0 THEN 'CANNOT_DETERMINE'
      ELSE test_case
    END AS test_case,
    result,
    build_id,
    timestamp
  FROM
    `grpc-testing.jenkins_test_results.rbe_test_results`
  WHERE
    DATETIME_DIFF(CURRENT_DATETIME(),
      dateTIME(timestamp),
      HOUR) < {lookback_hours}
    ),

  results_counts_per_build AS (
  SELECT
    test_binary,
    #test_target, # aggregate data over all pollers
    test_class_name,
    test_case,
    SUM(SAFE_CAST(result != 'PASSED'
        AND result != 'SKIPPED' AS INT64)) AS runs_failed,
    SUM(SAFE_CAST(result != 'SKIPPED' AS INT64)) AS runs_total,
    job_name,
    build_id
  FROM
    runs
  GROUP BY
    test_binary,
    test_class_name,
    test_case,
    job_name,
    build_id),
    
  builds_with_missing_cannot_determine_testcase_entry AS (
    SELECT 
      test_binary,
      job_name,
      build_id,
    FROM
      results_counts_per_build
    GROUP BY
      test_binary,
      job_name,
      build_id
    HAVING COUNTIF(test_case = 'CANNOT_DETERMINE') = 0
  ),
  
  # for each test target and build, generate a fake entry with "CANNOT_DETERMINE" test_case
  # if not already present.
  # this is because in many builds, there will be no "CANNOT_DETERMINE" entry
  # and we want to avoid skewing the statistics
  results_counts_per_build_with_fake_cannot_determine_test_case_entries AS (
    (SELECT * FROM results_counts_per_build)
    UNION ALL
    (SELECT
      test_binary,
      '' AS test_class_name,  # when test_case is 'CANNOT_DETERMINE', test class is empty string
      'CANNOT_DETERMINE' AS test_case,  # see table "runs"
      0 AS runs_failed,
      1 AS runs_total,
      job_name,
      build_id
    FROM
      builds_with_missing_cannot_determine_testcase_entry)
  ),
    
  results_counts AS (
  SELECT
    test_binary,
    test_class_name,
    test_case,
    job_name,
    SUM(runs_failed) AS runs_failed,
    SUM(runs_total) AS runs_total,
    SUM(SAFE_CAST(runs_failed > 0 AS INT64)) AS builds_failed,
    COUNT(build_id) AS builds_total,
    STRING_AGG(CASE
        WHEN runs_failed > 0 THEN 'X'
        ELSE '_' END, ''
    ORDER BY
      build_id ASC) AS build_failure_pattern,
    FORMAT("%T", ARRAY_AGG(build_id
      ORDER BY
        build_id ASC)) AS builds
  FROM
    #results_counts_per_build
    results_counts_per_build_with_fake_cannot_determine_test_case_entries
  GROUP BY
    test_binary,
    test_class_name,
    test_case,
    job_name
  HAVING
    runs_failed > 0)

SELECT
  ROUND(100*builds_failed / builds_total, 2) AS pct_builds_failed,
  ROUND(100*runs_failed / runs_total, 2) AS pct_runs_failed,
  test_binary,
  test_class_name,
  test_case,
  job_name,
  build_failure_pattern
           
FROM
  results_counts
ORDER BY
  pct_builds_failed DESC
"""



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/type/http/v3/cookie.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/type/http/v3/cookie.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1f\x65nvoy/type/http/v3/cookie.proto\x12\x12\x65nvoy.type.http.v3\x1a\x1egoogle/protobuf/duration.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\x98\x01\n\x06\x43ookie\x12\x15\n\x04name\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\x12\x30\n\x03ttl\x18\x02 \x01(\x0b\x32\x19.google.protobuf.DurationB\x08\xfa\x42\x05\xaa\x01\x02\x32\x00\x12\x0c\n\x04path\x18\x03 \x01(\t\x12\x37\n\nattributes\x18\x04 \x03(\x0b\x32#.envoy.type.http.v3.CookieAttribute\"R\n\x0f\x43ookieAttribute\x12\x1f\n\x04name\x18\x01 \x01(\tB\x11\xfa\x42\x0er\x0c\x10\x01(\x80\x80\x01\xc0\x01\x01\xc8\x01\x00\x12\x1e\n\x05value\x18\x02 \x01(\tB\x0f\xfa\x42\x0cr\n(\x80\x80\x01\xc0\x01\x02\xc8\x01\x00\x42{\n io.envoyproxy.envoy.type.http.v3B\x0b\x43ookieProtoP\x01Z@github.com/envoyproxy/go-control-plane/envoy/type/http/v3;httpv3\xba\x80\xc8\xd1\x06\x02\x10\x02\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.type.http.v3.cookie_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n io.envoyproxy.envoy.type.http.v3B\013CookieProtoP\001Z@github.com/envoyproxy/go-control-plane/envoy/type/http/v3;httpv3\272\200\310\321\006\002\020\002'
  _globals['_COOKIE'].fields_by_name['name']._loaded_options = None
  _globals['_COOKIE'].fields_by_name['name']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_COOKIE'].fields_by_name['ttl']._loaded_options = None
  _globals['_COOKIE'].fields_by_name['ttl']._serialized_options = b'\372B\005\252\001\0022\000'
  _globals['_COOKIEATTRIBUTE'].fields_by_name['name']._loaded_options = None
  _globals['_COOKIEATTRIBUTE'].fields_by_name['name']._serialized_options = b'\372B\016r\014\020\001(\200\200\001\300\001\001\310\001\000'
  _globals['_COOKIEATTRIBUTE'].fields_by_name['value']._loaded_options = None
  _globals['_COOKIEATTRIBUTE'].fields_by_name['value']._serialized_options = b'\372B\014r\n(\200\200\001\300\001\002\310\001\000'
  _globals['_COOKIE']._serialized_start=144
  _globals['_COOKIE']._serialized_end=296
  _globals['_COOKIEATTRIBUTE']._serialized_start=298
  _globals['_COOKIEATTRIBUTE']._serialized_end=380
# @@protoc_insertion_point(module_scope)



# Copyright 2016 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests server and client side compression."""

from concurrent import futures
import contextlib
import functools
import itertools
import logging
import os
import unittest

import grpc
from grpc import _grpcio_metadata

from tests.unit import _tcp_proxy
from tests.unit.framework.common import test_constants

_SERVICE_NAME = "test"
_UNARY_UNARY = "UnaryUnary"
_UNARY_STREAM = "UnaryStream"
_STREAM_UNARY = "StreamUnary"
_STREAM_STREAM = "StreamStream"

# Cut down on test time.
_STREAM_LENGTH = test_constants.STREAM_LENGTH // 16

_HOST = "localhost"

_REQUEST = b"\x00" * 100
_COMPRESSION_RATIO_THRESHOLD = 0.05
_COMPRESSION_METHODS = (
    None,
    # Disabled for test tractability.
    # grpc.Compression.NoCompression,
    # grpc.Compression.Deflate,
    grpc.Compression.Gzip,
)
_COMPRESSION_NAMES = {
    None: "Uncompressed",
    grpc.Compression.NoCompression: "NoCompression",
    grpc.Compression.Deflate: "DeflateCompression",
    grpc.Compression.Gzip: "GzipCompression",
}

_TEST_OPTIONS = {
    "client_streaming": (True, False),
    "server_streaming": (True, False),
    "channel_compression": _COMPRESSION_METHODS,
    "multicallable_compression": _COMPRESSION_METHODS,
    "server_compression": _COMPRESSION_METHODS,
    "server_call_compression": _COMPRESSION_METHODS,
}


def _make_handle_unary_unary(pre_response_callback):
    def _handle_unary(request, servicer_context):
        if pre_response_callback:
            pre_response_callback(request, servicer_context)
        return request

    return _handle_unary


def _make_handle_unary_stream(pre_response_callback):
    def _handle_unary_stream(request, servicer_context):
        if pre_response_callback:
            pre_response_callback(request, servicer_context)
        for _ in range(_STREAM_LENGTH):
            yield request

    return _handle_unary_stream


def _make_handle_stream_unary(pre_response_callback):
    def _handle_stream_unary(request_iterator, servicer_context):
        if pre_response_callback:
            pre_response_callback(request_iterator, servicer_context)
        response = None
        for request in request_iterator:
            if not response:
                response = request
        return response

    return _handle_stream_unary


def _make_handle_stream_stream(pre_response_callback):
    def _handle_stream(request_iterator, servicer_context):
        # TODO(issue:#6891) We should be able to remove this loop,
        # and replace with return; yield
        for request in request_iterator:
            if pre_response_callback:
                pre_response_callback(request, servicer_context)
            yield request

    return _handle_stream


def set_call_compression(
    compression_method, request_or_iterator, servicer_context
):
    del request_or_iterator
    servicer_context.set_compression(compression_method)


def disable_next_compression(request, servicer_context):
    del request
    servicer_context.disable_next_message_compression()


def disable_first_compression(request, servicer_context):
    if int(request.decode("ascii")) == 0:
        servicer_context.disable_next_message_compression()


class _MethodHandler(grpc.RpcMethodHandler):
    def __init__(
        self, request_streaming, response_streaming, pre_response_callback
    ):
        self.request_streaming = request_streaming
        self.response_streaming = response_streaming
        self.request_deserializer = None
        self.response_serializer = None
        self.unary_unary = None
        self.unary_stream = None
        self.stream_unary = None
        self.stream_stream = None

        if self.request_streaming and self.response_streaming:
            self.stream_stream = _make_handle_stream_stream(
                pre_response_callback
            )
        elif not self.request_streaming and not self.response_streaming:
            self.unary_unary = _make_handle_unary_unary(pre_response_callback)
        elif not self.request_streaming and self.response_streaming:
            self.unary_stream = _make_handle_unary_stream(pre_response_callback)
        else:
            self.stream_unary = _make_handle_stream_unary(pre_response_callback)


def get_method_handlers(pre_response_callback):
    return {
        _UNARY_UNARY: _MethodHandler(False, False, pre_response_callback),
        _UNARY_STREAM: _MethodHandler(False, True, pre_response_callback),
        _STREAM_UNARY: _MethodHandler(True, False, pre_response_callback),
        _STREAM_STREAM: _MethodHandler(True, True, pre_response_callback),
    }


@contextlib.contextmanager
def _instrumented_client_server_pair(
    channel_kwargs, server_kwargs, server_handler
):
    server = grpc.server(futures.ThreadPoolExecutor(), **server_kwargs)
    server.add_registered_method_handlers(_SERVICE_NAME, server_handler)
    server_port = server.add_insecure_port("{}:0".format(_HOST))
    server.start()
    with _tcp_proxy.TcpProxy(_HOST, _HOST, server_port) as proxy:
        proxy_port = proxy.get_port()
        with grpc.insecure_channel(
            "{}:{}".format(_HOST, proxy_port), **channel_kwargs
        ) as client_channel:
            try:
                yield client_channel, proxy, server
            finally:
                server.stop(None)


def _get_byte_counts(
    channel_kwargs,
    multicallable_kwargs,
    client_function,
    server_kwargs,
    server_handler,
    message,
):
    with _instrumented_client_server_pair(
        channel_kwargs, server_kwargs, server_handler
    ) as pipeline:
        client_channel, proxy, server = pipeline
        client_function(client_channel, multicallable_kwargs, message)
        return proxy.get_byte_count()


def _get_compression_ratios(
    client_function,
    first_channel_kwargs,
    first_multicallable_kwargs,
    first_server_kwargs,
    first_server_handler,
    second_channel_kwargs,
    second_multicallable_kwargs,
    second_server_kwargs,
    second_server_handler,
    message,
):
    first_bytes_sent, first_bytes_received = _get_byte_counts(
        first_channel_kwargs,
        first_multicallable_kwargs,
        client_function,
        first_server_kwargs,
        first_server_handler,
        message,
    )
    second_bytes_sent, second_bytes_received = _get_byte_counts(
        second_channel_kwargs,
        second_multicallable_kwargs,
        client_function,
        second_server_kwargs,
        second_server_handler,
        message,
    )
    return (
        (second_bytes_sent - first_bytes_sent) / float(first_bytes_sent),
        (second_bytes_received - first_bytes_received)
        / float(first_bytes_received),
    )


def _unary_unary_client(channel, multicallable_kwargs, message):
    multi_callable = channel.unary_unary(
        grpc._common.fully_qualified_method(_SERVICE_NAME, _UNARY_UNARY),
        _registered_method=True,
    )
    response = multi_callable(message, **multicallable_kwargs)
    if response != message:
        raise RuntimeError(
            "Request '{}' != Response '{}'".format(message, response)
        )


def _unary_stream_client(channel, multicallable_kwargs, message):
    multi_callable = channel.unary_stream(
        grpc._common.fully_qualified_method(_SERVICE_NAME, _UNARY_STREAM),
        _registered_method=True,
    )
    response_iterator = multi_callable(message, **multicallable_kwargs)
    for response in response_iterator:
        if response != message:
            raise RuntimeError(
                "Request '{}' != Response '{}'".format(message, response)
            )


def _stream_unary_client(channel, multicallable_kwargs, message):
    multi_callable = channel.stream_unary(
        grpc._common.fully_qualified_method(_SERVICE_NAME, _STREAM_UNARY),
        _registered_method=True,
    )
    requests = (_REQUEST for _ in range(_STREAM_LENGTH))
    response = multi_callable(requests, **multicallable_kwargs)
    if response != message:
        raise RuntimeError(
            "Request '{}' != Response '{}'".format(message, response)
        )


def _stream_stream_client(channel, multicallable_kwargs, message):
    multi_callable = channel.stream_stream(
        grpc._common.fully_qualified_method(_SERVICE_NAME, _STREAM_STREAM),
        _registered_method=True,
    )
    request_prefix = str(0).encode("ascii") * 100
    requests = (
        request_prefix + str(i).encode("ascii") for i in range(_STREAM_LENGTH)
    )
    response_iterator = multi_callable(requests, **multicallable_kwargs)
    for i, response in enumerate(response_iterator):
        if int(response.decode("ascii")) != i:
            raise RuntimeError(
                "Request '{}' != Response '{}'".format(i, response)
            )


class CompressionTest(unittest.TestCase):
    def assertCompressed(self, compression_ratio):
        self.assertLess(
            compression_ratio,
            -1.0 * _COMPRESSION_RATIO_THRESHOLD,
            msg="Actual compression ratio: {}".format(compression_ratio),
        )

    def assertNotCompressed(self, compression_ratio):
        self.assertGreaterEqual(
            compression_ratio,
            -1.0 * _COMPRESSION_RATIO_THRESHOLD,
            msg="Actual compression ratio: {}".format(compression_ratio),
        )

    def assertConfigurationCompressed(
        self,
        client_streaming,
        server_streaming,
        channel_compression,
        multicallable_compression,
        server_compression,
        server_call_compression,
    ):
        client_side_compressed = (
            channel_compression or multicallable_compression
        )
        server_side_compressed = server_compression or server_call_compression
        channel_kwargs = (
            {
                "compression": channel_compression,
            }
            if channel_compression
            else {}
        )
        multicallable_kwargs = (
            {
                "compression": multicallable_compression,
            }
            if multicallable_compression
            else {}
        )

        client_function = None
        if not client_streaming and not server_streaming:
            client_function = _unary_unary_client
        elif not client_streaming and server_streaming:
            client_function = _unary_stream_client
        elif client_streaming and not server_streaming:
            client_function = _stream_unary_client
        else:
            client_function = _stream_stream_client

        server_kwargs = (
            {
                "compression": server_compression,
            }
            if server_compression
            else {}
        )
        if server_call_compression:
            server_handler = get_method_handlers(
                functools.partial(set_call_compression, grpc.Compression.Gzip)
            )
        else:
            server_handler = get_method_handlers(None)

        _get_compression_ratios(
            client_function,
            {},
            {},
            {},
            get_method_handlers(None),
            channel_kwargs,
            multicallable_kwargs,
            server_kwargs,
            server_handler,
            _REQUEST,
        )

    def testDisableNextCompressionStreaming(self):
        server_kwargs = {
            "compression": grpc.Compression.Deflate,
        }
        _get_compression_ratios(
            _stream_stream_client,
            {},
            {},
            {},
            get_method_handlers(None),
            {},
            {},
            server_kwargs,
            get_method_handlers(disable_next_compression),
            _REQUEST,
        )

    def testDisableNextCompressionStreamingResets(self):
        server_kwargs = {
            "compression": grpc.Compression.Deflate,
        }
        _get_compression_ratios(
            _stream_stream_client,
            {},
            {},
            {},
            get_method_handlers(None),
            {},
            {},
            server_kwargs,
            get_method_handlers(disable_first_compression),
            _REQUEST,
        )


def _get_compression_str(name, value):
    return "{}{}".format(name, _COMPRESSION_NAMES[value])


def _get_compression_test_name(
    client_streaming,
    server_streaming,
    channel_compression,
    multicallable_compression,
    server_compression,
    server_call_compression,
):
    client_arity = "Stream" if client_streaming else "Unary"
    server_arity = "Stream" if server_streaming else "Unary"
    arity = "{}{}".format(client_arity, server_arity)
    channel_compression_str = _get_compression_str(
        "Channel", channel_compression
    )
    multicallable_compression_str = _get_compression_str(
        "Multicallable", multicallable_compression
    )
    server_compression_str = _get_compression_str("Server", server_compression)
    server_call_compression_str = _get_compression_str(
        "ServerCall", server_call_compression
    )
    return "test{}{}{}{}{}".format(
        arity,
        channel_compression_str,
        multicallable_compression_str,
        server_compression_str,
        server_call_compression_str,
    )


def _test_options():
    for test_parameters in itertools.product(*_TEST_OPTIONS.values()):
        yield dict(zip(_TEST_OPTIONS.keys(), test_parameters))


for options in _test_options():

    def test_compression(**kwargs):
        def _test_compression(self):
            self.assertConfigurationCompressed(**kwargs)

        return _test_compression

    setattr(
        CompressionTest,
        _get_compression_test_name(**options),
        test_compression(**options),
    )

if __name__ == "__main__":
    logging.basicConfig()
    unittest.main(verbosity=2)



# Copyright 2022 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import functions_framework
from google.cloud import pubsub_v1

ps_client = pubsub_v1.PublisherClient()
_PROJECT_ID = "grpc-testing"
_PUBSUB_TOPIC = "gcf-distribtest-topic"


@functions_framework.http
def test_publish(request):
    topic_path = ps_client.topic_path(_PROJECT_ID, _PUBSUB_TOPIC)
    message = '{"function": "TEST"}'
    message_bytes = message.encode("utf-8")

    for _ in range(100):
        future = ps_client.publish(topic_path, data=message_bytes)

    return "ok", 200



# Copyright 2023 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from datetime import datetime
import os
from typing import List, Mapping, Optional, Tuple

from google.rpc import code_pb2
from grpc_observability import _observability
from grpc_observability import _observability_config
from grpc_observability import _views
from opencensus.common.transports import async_
from opencensus.ext.stackdriver import stats_exporter
from opencensus.ext.stackdriver import trace_exporter
from opencensus.stats import stats as stats_module
from opencensus.stats.stats_recorder import StatsRecorder
from opencensus.stats.view_manager import ViewManager
from opencensus.tags.tag_key import TagKey
from opencensus.tags.tag_map import TagMap
from opencensus.tags.tag_value import TagValue
from opencensus.trace import execution_context
from opencensus.trace import samplers
from opencensus.trace import span
from opencensus.trace import span_context as span_context_module
from opencensus.trace import span_data as span_data_module
from opencensus.trace import status
from opencensus.trace import time_event
from opencensus.trace import trace_options
from opencensus.trace import tracer

# 60s is the default time for open census to call export.
CENSUS_UPLOAD_INTERVAL_SECS = int(
    os.environ.get("GRPC_PYTHON_CENSUS_EXPORT_UPLOAD_INTERVAL_SECS", "20")
)


class StackDriverAsyncTransport(async_.AsyncTransport):
    """Wrapper class used to pass wait_period.

    This is required because current StackDriver Tracing Exporter doesn't allow
    us pass wait_period to AsyncTransport directly.

    Args:
      exporter: An opencensus.trace.base_exporter.Exporter object.
    """

    def __init__(self, exporter):
        super().__init__(exporter, wait_period=CENSUS_UPLOAD_INTERVAL_SECS)


class OpenCensusExporter(_observability.Exporter):
    config: _observability_config.GcpObservabilityConfig
    default_labels: Optional[Mapping[str, str]]
    project_id: str
    tracer: Optional[tracer.Tracer]
    stats_recorder: Optional[StatsRecorder]
    view_manager: Optional[ViewManager]

    def __init__(self, config: _observability_config.GcpObservabilityConfig):
        self.config = config.get()
        self.default_labels = self.config.labels
        self.project_id = self.config.project_id
        self.tracer = None
        self.stats_recorder = None
        self.view_manager = None
        self._setup_open_census_stackdriver_exporter()

    def _setup_open_census_stackdriver_exporter(self) -> None:
        if self.config.stats_enabled:
            stats = stats_module.stats
            self.stats_recorder = stats.stats_recorder
            self.view_manager = stats.view_manager
            # If testing locally please add resource="global" to Options, otherwise
            # StackDriver might override project_id based on detected resource.
            options = stats_exporter.Options(project_id=self.project_id)
            metrics_exporter = stats_exporter.new_stats_exporter(
                options, interval=CENSUS_UPLOAD_INTERVAL_SECS
            )
            self.view_manager.register_exporter(metrics_exporter)
            self._register_open_census_views()

        if self.config.tracing_enabled:
            current_tracer = execution_context.get_opencensus_tracer()
            trace_id = current_tracer.span_context.trace_id
            span_id = current_tracer.span_context.span_id
            if not span_id:
                span_id = span_context_module.generate_span_id()
            span_context = span_context_module.SpanContext(
                trace_id=trace_id, span_id=span_id
            )
            # Create and Saves Tracer and Sampler to ContextVar
            sampler = samplers.ProbabilitySampler(
                rate=self.config.sampling_rate
            )
            self.trace_exporter = trace_exporter.StackdriverExporter(
                project_id=self.project_id,
                transport=StackDriverAsyncTransport,
            )
            self.tracer = tracer.Tracer(
                sampler=sampler,
                span_context=span_context,
                exporter=self.trace_exporter,
            )

    def export_stats_data(
        self, stats_data: List[_observability.StatsData]
    ) -> None:
        if not self.config.stats_enabled:
            return
        for data in stats_data:
            measure = _views.METRICS_NAME_TO_MEASURE.get(data.name, None)
            if not measure:
                continue
            # Create a measurement map for each metric, otherwise metrics will
            # be overridden instead of accumulate.
            measurement_map = self.stats_recorder.new_measurement_map()
            # Add data label to default labels.
            labels = data.labels
            labels.update(self.default_labels)
            tag_map = TagMap()
            for key, value in labels.items():
                tag_map.insert(TagKey(key), TagValue(value))

            if data.measure_double:
                measurement_map.measure_float_put(measure, data.value_float)
            else:
                measurement_map.measure_int_put(measure, data.value_int)
            measurement_map.record(tag_map)

    def export_tracing_data(
        self, tracing_data: List[_observability.TracingData]
    ) -> None:
        if not self.config.tracing_enabled:
            return
        for span_data in tracing_data:
            # Only traced data will be exported, thus TraceOptions=1.
            span_context = span_context_module.SpanContext(
                trace_id=span_data.trace_id,
                span_id=span_data.span_id,
                trace_options=trace_options.TraceOptions(1),
            )
            span_datas = _get_span_data(
                span_data, span_context, self.default_labels
            )
            self.trace_exporter.export(span_datas)

    def _register_open_census_views(self) -> None:
        # Client
        self.view_manager.register_view(
            _views.client_started_rpcs(self.default_labels)
        )
        self.view_manager.register_view(
            _views.client_completed_rpcs(self.default_labels)
        )
        self.view_manager.register_view(
            _views.client_roundtrip_latency(self.default_labels)
        )
        self.view_manager.register_view(
            _views.client_api_latency(self.default_labels)
        )
        self.view_manager.register_view(
            _views.client_sent_compressed_message_bytes_per_rpc(
                self.default_labels
            )
        )
        self.view_manager.register_view(
            _views.client_received_compressed_message_bytes_per_rpc(
                self.default_labels
            )
        )

        # Server
        self.view_manager.register_view(
            _views.server_started_rpcs(self.default_labels)
        )
        self.view_manager.register_view(
            _views.server_completed_rpcs(self.default_labels)
        )
        self.view_manager.register_view(
            _views.server_sent_compressed_message_bytes_per_rpc(
                self.default_labels
            )
        )
        self.view_manager.register_view(
            _views.server_received_compressed_message_bytes_per_rpc(
                self.default_labels
            )
        )
        self.view_manager.register_view(
            _views.server_server_latency(self.default_labels)
        )


def _get_span_annotations(
    span_annotations: List[Tuple[str, str]],
) -> List[time_event.Annotation]:
    annotations = []

    for time_stamp, description in span_annotations:
        time = datetime.fromisoformat(time_stamp)
        annotations.append(time_event.Annotation(time, description))

    return annotations


# pylint: disable=too-many-return-statements
# pylint: disable=too-many-branches
def _status_to_span_status(  # noqa: PLR0911
    span_status: str,
) -> Optional[status.Status]:
    if status == "OK":
        return status.Status(code_pb2.OK, message=span_status)
    if status == "CANCELLED":
        return status.Status(code_pb2.CANCELLED, message=span_status)
    if status == "UNKNOWN":
        return status.Status(code_pb2.UNKNOWN, message=span_status)
    if status == "INVALID_ARGUMENT":
        return status.Status(code_pb2.INVALID_ARGUMENT, message=span_status)
    if status == "DEADLINE_EXCEEDED":
        return status.Status(code_pb2.DEADLINE_EXCEEDED, message=span_status)
    if status == "NOT_FOUND":
        return status.Status(code_pb2.NOT_FOUND, message=span_status)
    if status == "ALREADY_EXISTS":
        return status.Status(code_pb2.ALREADY_EXISTS, message=span_status)
    if status == "PERMISSION_DENIED":
        return status.Status(code_pb2.PERMISSION_DENIED, message=span_status)
    if status == "UNAUTHENTICATED":
        return status.Status(code_pb2.UNAUTHENTICATED, message=span_status)
    if status == "RESOURCE_EXHAUSTED":
        return status.Status(code_pb2.RESOURCE_EXHAUSTED, message=span_status)
    if status == "FAILED_PRECONDITION":
        return status.Status(code_pb2.FAILED_PRECONDITION, message=span_status)
    if status == "ABORTED":
        return status.Status(code_pb2.ABORTED, message=span_status)
    if status == "OUT_OF_RANGE":
        return status.Status(code_pb2.OUT_OF_RANGE, message=span_status)
    if status == "UNIMPLEMENTED":
        return status.Status(code_pb2.UNIMPLEMENTED, message=span_status)
    if status == "INTERNAL":
        return status.Status(code_pb2.INTERNAL, message=span_status)
    if status == "UNAVAILABLE":
        return status.Status(code_pb2.UNAVAILABLE, message=span_status)
    if status == "DATA_LOSS":
        return status.Status(code_pb2.DATA_LOSS, message=span_status)
    return None


def _get_span_data(
    span_data: _observability.TracingData,
    span_context: span_context_module.SpanContext,
    labels: Mapping[str, str],
) -> List[span_data_module.SpanData]:
    """Extracts a list of SpanData tuples from a span.

    Args:
    span_data: _observability.TracingData to convert.
    span_context: The context related to the span_data.
    labels: Labels to be added to SpanData.

    Returns:
    A list of opencensus.trace.span_data.SpanData.
    """
    span_attributes = span_data.span_labels
    span_attributes.update(labels)
    span_status = _status_to_span_status(span_data.status)
    span_annotations = _get_span_annotations(span_data.span_annotations)
    span_datas = [
        span_data_module.SpanData(
            name=span_data.name,
            context=span_context,
            span_id=span_data.span_id,
            parent_span_id=(
                span_data.parent_span_id if span_data.parent_span_id else None
            ),
            attributes=span_attributes,
            start_time=span_data.start_time,
            end_time=span_data.end_time,
            child_span_count=span_data.child_span_count,
            stack_trace=None,
            annotations=span_annotations,
            message_events=None,
            links=None,
            status=span_status,
            same_process_as_parent_span=(
                True if span_data.parent_span_id else None
            ),
            span_kind=span.SpanKind.UNSPECIFIED,
        )
    ]

    return span_datas



# Copyright 2026 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""gRPC Python AsyncIO helloworld.Greeter client with observability enabled."""

import asyncio
import collections
import logging

import grpc
import grpc_observability
import helloworld_pb2
import helloworld_pb2_grpc
import open_telemetry_exporter
from opentelemetry.sdk import metrics as otel_metrics
from opentelemetry.sdk.metrics import export as otel_metrics_export

OTEL_EXPORT_INTERVAL_S = 0.5


async def run() -> None:
    all_metrics = collections.defaultdict(list)
    otel_exporter = open_telemetry_exporter.OTelMetricExporter(all_metrics)
    reader = otel_metrics_export.PeriodicExportingMetricReader(
        exporter=otel_exporter,
        export_interval_millis=OTEL_EXPORT_INTERVAL_S * 1000,
    )
    provider = otel_metrics.MeterProvider(metric_readers=(reader,))

    otel_plugin = grpc_observability.OpenTelemetryPlugin(
        meter_provider=provider
    )
    otel_plugin.register_global()

    async with grpc.aio.insecure_channel(target="localhost:50051") as channel:
        stub = helloworld_pb2_grpc.GreeterStub(channel)
        try:
            response = await stub.SayHello(
                helloworld_pb2.HelloRequest(name="AsyncWorld")
            )
            print(f"Greeter client received: {response.message}")
        except grpc.RpcError as rpc_error:
            print("Call failed with code: ", rpc_error.code())
    otel_plugin.deregister_global()

    # Shutdown the provider to force a flush of all metrics.
    provider.shutdown()

    print("Metrics exported on client side:")
    for metric in all_metrics:
        print(metric)


if __name__ == "__main__":
    logging.basicConfig()
    asyncio.run(run())



# Copyright 2026 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import os
import platform
import subprocess
import sys
import tempfile

_DEBUGGER_TIMEOUT_S = 60


def print_backtraces(pid):
    # Check if we are on macOS AND the environment variable is set
    use_lldb = False
    if platform.system() == "Darwin":
        use_lldb = bool(os.environ.get("USE_LLDB_ON_DARWIN"))
        if not use_lldb:
            sys.stderr.write(
                "LLDB is recommended for use on Mac.\n"
                + 'Run "sudo DevToolsSecurity -enable" and set USE_LLDB_ON_DARWIN'
                + " environment variable to use lldb\n"
            )
    # Determine the OS and set debugger specific commands
    if use_lldb:  # macOS
        debugger_name = "lldb"
        cmd = [
            "lldb",
            "--no-lldbinit",
            "-b",  # Batch mode: exit after commands are executed
            "-o",
            "process attach --pid {}".format(pid),
            "-o",
            "thread backtrace all",  # Dumps backtraces of all threads
            "-o",
            "quit",
        ]
    else:  # Assume Linux/Unix-like for gdb (adjust if other OSes need specific handling)
        debugger_name = "gdb"
        cmd = [
            "gdb",
            "-ex",
            "set confirm off",
            "-ex",
            "attach {}".format(pid),
            "-ex",
            "thread apply all bt",
            "-ex",
            "quit",
        ]
    streams = tuple(tempfile.TemporaryFile() for _ in range(2))
    sys.stderr.write(f"Invoking {debugger_name}\n")
    sys.stderr.flush()
    process = subprocess.Popen(cmd, stdout=streams[0], stderr=streams[1])
    try:
        process.wait(timeout=_DEBUGGER_TIMEOUT_S)
    except subprocess.TimeoutExpired:
        sys.stderr.write(
            "{} stacktrace generation timed out.\n".format(debugger_name)
        )
    finally:
        for stream_name, stream in zip(("STDOUT", "STDERR"), streams):
            stream.seek(0)
            sys.stderr.write(
                "{} {}:\n{}\n".format(
                    debugger_name, stream_name, stream.read().decode("ascii")
                )
            )
            stream.close()
        sys.stderr.flush()



# Copyright 2015 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Utilities for testing stream-related code."""

from grpc.framework.foundation import stream


class TestConsumer(stream.Consumer):
    """A stream.Consumer instrumented for testing.

    Attributes:
      calls: A sequence of value-termination pairs describing the history of calls
        made on this object.
    """

    def __init__(self):
        self.calls = []

    def consume(self, value):
        """See stream.Consumer.consume for specification."""
        self.calls.append((value, False))

    def terminate(self):
        """See stream.Consumer.terminate for specification."""
        self.calls.append((None, True))

    def consume_and_terminate(self, value):
        """See stream.Consumer.consume_and_terminate for specification."""
        self.calls.append((value, True))

    def is_legal(self):
        """Reports whether or not a legal sequence of calls has been made."""
        terminated = False
        for value, terminal in self.calls:
            if terminated:
                return False
            elif terminal:
                terminated = True
            elif value is None:
                return False
        else:  # pylint: disable=useless-else-on-loop
            return True

    def values(self):
        """Returns the sequence of values that have been passed to this Consumer."""
        return [value for value, _ in self.calls if value]



# Copyright 2017 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import threading

import grpc_testing
from grpc_testing import _common
from grpc_testing._server import _handler
from grpc_testing._server import _rpc
from grpc_testing._server import _server_rpc
from grpc_testing._server import _service
from grpc_testing._server import _servicer_context


def _implementation(descriptors_to_servicers, method_descriptor):
    servicer = descriptors_to_servicers[method_descriptor.containing_service]
    return getattr(servicer, method_descriptor.name)


def _unary_unary_service(request):
    def service(implementation, rpc, servicer_context):
        _service.unary_unary(implementation, rpc, request, servicer_context)

    return service


def _unary_stream_service(request):
    def service(implementation, rpc, servicer_context):
        _service.unary_stream(implementation, rpc, request, servicer_context)

    return service


def _stream_unary_service(handler):
    def service(implementation, rpc, servicer_context):
        _service.stream_unary(implementation, rpc, handler, servicer_context)

    return service


def _stream_stream_service(handler):
    def service(implementation, rpc, servicer_context):
        _service.stream_stream(implementation, rpc, handler, servicer_context)

    return service


class _Serverish(_common.Serverish):
    def __init__(self, descriptors_to_servicers, time):
        self._descriptors_to_servicers = descriptors_to_servicers
        self._time = time

    def _invoke(
        self,
        service_behavior,
        method_descriptor,
        handler,
        invocation_metadata,
        deadline,
    ):
        implementation = _implementation(
            self._descriptors_to_servicers, method_descriptor
        )
        rpc = _rpc.Rpc(handler, invocation_metadata)
        if handler.add_termination_callback(rpc.extrinsic_abort):
            servicer_context = _servicer_context.ServicerContext(
                rpc, self._time, deadline
            )
            service_thread = threading.Thread(
                target=service_behavior,
                args=(
                    implementation,
                    rpc,
                    servicer_context,
                ),
            )
            service_thread.start()

    def invoke_unary_unary(
        self, method_descriptor, handler, invocation_metadata, request, deadline
    ):
        self._invoke(
            _unary_unary_service(request),
            method_descriptor,
            handler,
            invocation_metadata,
            deadline,
        )

    def invoke_unary_stream(
        self, method_descriptor, handler, invocation_metadata, request, deadline
    ):
        self._invoke(
            _unary_stream_service(request),
            method_descriptor,
            handler,
            invocation_metadata,
            deadline,
        )

    def invoke_stream_unary(
        self, method_descriptor, handler, invocation_metadata, deadline
    ):
        self._invoke(
            _stream_unary_service(handler),
            method_descriptor,
            handler,
            invocation_metadata,
            deadline,
        )

    def invoke_stream_stream(
        self, method_descriptor, handler, invocation_metadata, deadline
    ):
        self._invoke(
            _stream_stream_service(handler),
            method_descriptor,
            handler,
            invocation_metadata,
            deadline,
        )


def _deadline_and_handler(requests_closed, time, timeout):
    if timeout is None:
        return None, _handler.handler_without_deadline(requests_closed)
    deadline = time.time() + timeout
    handler = _handler.handler_with_deadline(requests_closed, time, deadline)
    return deadline, handler


class _Server(grpc_testing.Server):
    def __init__(self, serverish, time):
        self._serverish = serverish
        self._time = time

    def invoke_unary_unary(
        self, method_descriptor, invocation_metadata, request, timeout
    ):
        deadline, handler = _deadline_and_handler(True, self._time, timeout)
        self._serverish.invoke_unary_unary(
            method_descriptor, handler, invocation_metadata, request, deadline
        )
        return _server_rpc.UnaryUnaryServerRpc(handler)

    def invoke_unary_stream(
        self, method_descriptor, invocation_metadata, request, timeout
    ):
        deadline, handler = _deadline_and_handler(True, self._time, timeout)
        self._serverish.invoke_unary_stream(
            method_descriptor, handler, invocation_metadata, request, deadline
        )
        return _server_rpc.UnaryStreamServerRpc(handler)

    def invoke_stream_unary(
        self, method_descriptor, invocation_metadata, timeout
    ):
        deadline, handler = _deadline_and_handler(False, self._time, timeout)
        self._serverish.invoke_stream_unary(
            method_descriptor, handler, invocation_metadata, deadline
        )
        return _server_rpc.StreamUnaryServerRpc(handler)

    def invoke_stream_stream(
        self, method_descriptor, invocation_metadata, timeout
    ):
        deadline, handler = _deadline_and_handler(False, self._time, timeout)
        self._serverish.invoke_stream_stream(
            method_descriptor, handler, invocation_metadata, deadline
        )
        return _server_rpc.StreamStreamServerRpc(handler)


def server_from_descriptor_to_servicers(descriptors_to_servicers, time):
    return _Server(_Serverish(descriptors_to_servicers, time), time)



# Copyright 2015 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Common code used throughout tests of gRPC."""

import ast
import collections
from concurrent import futures
import os
import threading

import grpc

INVOCATION_INITIAL_METADATA = (
    ("0", "abc"),
    ("1", "def"),
    ("2", "ghi"),
)
SERVICE_INITIAL_METADATA = (
    ("3", "jkl"),
    ("4", "mno"),
    ("5", "pqr"),
)
SERVICE_TERMINAL_METADATA = (
    ("6", "stu"),
    ("7", "vwx"),
    ("8", "yza"),
)
DETAILS = "test details"


def metadata_transmitted(original_metadata, transmitted_metadata):
    """Judges whether or not metadata was acceptably transmitted.

    gRPC is allowed to insert key-value pairs into the metadata values given by
    applications and to reorder key-value pairs with different keys but it is not
    allowed to alter existing key-value pairs or to reorder key-value pairs with
    the same key.

    Args:
      original_metadata: A metadata value used in a test of gRPC. An iterable over
        iterables of length 2.
      transmitted_metadata: A metadata value corresponding to original_metadata
        after having been transmitted via gRPC. An iterable over iterables of
        length 2.

    Returns:
       A boolean indicating whether transmitted_metadata accurately reflects
        original_metadata after having been transmitted via gRPC.
    """
    original = collections.defaultdict(list)
    for key, value in original_metadata:
        original[key].append(value)
    transmitted = collections.defaultdict(list)
    for key, value in transmitted_metadata:
        transmitted[key].append(value)

    for key, values in original.items():
        transmitted_values = transmitted[key]
        transmitted_iterator = iter(transmitted_values)
        try:
            for value in values:
                while True:
                    transmitted_value = next(transmitted_iterator)
                    if value == transmitted_value:
                        break
        except StopIteration:
            return False
    else:
        return True


def test_secure_channel(target, channel_credentials, server_host_override):
    """Creates an insecure Channel to a remote host.

    Args:
      host: The name of the remote host to which to connect.
      port: The port of the remote host to which to connect.
      channel_credentials: The implementations.ChannelCredentials with which to
        connect.
      server_host_override: The target name used for SSL host name checking.

    Returns:
      An implementations.Channel to the remote host through which RPCs may be
        conducted.
    """
    channel = grpc.secure_channel(
        target,
        channel_credentials,
        (
            (
                "grpc.ssl_target_name_override",
                server_host_override,
            ),
        ),
    )
    return channel


def test_server(max_workers=10, reuse_port=False):
    """Creates an insecure grpc server.

    These servers have SO_REUSEPORT disabled to prevent cross-talk.
    """
    server_kwargs = os.environ.get("GRPC_ADDITIONAL_SERVER_KWARGS", "{}")
    server_kwargs = ast.literal_eval(server_kwargs)
    return grpc.server(
        futures.ThreadPoolExecutor(max_workers=max_workers),
        options=(("grpc.so_reuseport", int(reuse_port)),),
        **server_kwargs,
    )


class WaitGroup:
    def __init__(self, n=0):
        self.count = n
        self.cv = threading.Condition()

    def add(self, n):
        self.cv.acquire()
        self.count += n
        self.cv.release()

    def done(self):
        self.cv.acquire()
        self.count -= 1
        if self.count == 0:
            self.cv.notify_all()
        self.cv.release()

    def wait(self):
        self.cv.acquire()
        while self.count > 0:
            self.cv.wait()
        self.cv.release()


def running_under_gevent():
    try:
        from gevent import monkey
        import gevent.socket
    except ImportError:
        return False
    else:
        import socket

        return socket.socket is gevent.socket.socket



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/type/percent.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/type/percent.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x18\x65nvoy/type/percent.proto\x12\nenvoy.type\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"1\n\x07Percent\x12&\n\x05value\x18\x01 \x01(\x01\x42\x17\xfa\x42\x14\x12\x12\x19\x00\x00\x00\x00\x00\x00Y@)\x00\x00\x00\x00\x00\x00\x00\x00\"\xb3\x01\n\x11\x46ractionalPercent\x12\x11\n\tnumerator\x18\x01 \x01(\r\x12L\n\x0b\x64\x65nominator\x18\x02 \x01(\x0e\x32-.envoy.type.FractionalPercent.DenominatorTypeB\x08\xfa\x42\x05\x82\x01\x02\x10\x01\"=\n\x0f\x44\x65nominatorType\x12\x0b\n\x07HUNDRED\x10\x00\x12\x10\n\x0cTEN_THOUSAND\x10\x01\x12\x0b\n\x07MILLION\x10\x02\x42\x65\n\x18io.envoyproxy.envoy.typeB\x0cPercentProtoP\x01Z1github.com/envoyproxy/go-control-plane/envoy/type\xba\x80\xc8\xd1\x06\x02\x10\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.type.percent_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\030io.envoyproxy.envoy.typeB\014PercentProtoP\001Z1github.com/envoyproxy/go-control-plane/envoy/type\272\200\310\321\006\002\020\001'
  _globals['_PERCENT'].fields_by_name['value']._loaded_options = None
  _globals['_PERCENT'].fields_by_name['value']._serialized_options = b'\372B\024\022\022\031\000\000\000\000\000\000Y@)\000\000\000\000\000\000\000\000'
  _globals['_FRACTIONALPERCENT'].fields_by_name['denominator']._loaded_options = None
  _globals['_FRACTIONALPERCENT'].fields_by_name['denominator']._serialized_options = b'\372B\005\202\001\002\020\001'
  _globals['_PERCENT']._serialized_start=96
  _globals['_PERCENT']._serialized_end=145
  _globals['_FRACTIONALPERCENT']._serialized_start=148
  _globals['_FRACTIONALPERCENT']._serialized_end=327
  _globals['_FRACTIONALPERCENT_DENOMINATORTYPE']._serialized_start=266
  _globals['_FRACTIONALPERCENT_DENOMINATORTYPE']._serialized_end=327
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/config/core/v3/cel.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/config/core/v3/cel.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1e\x65nvoy/config/core/v3/cel.proto\x12\x14\x65nvoy.config.core.v3\x1a\x1dudpa/annotations/status.proto\"v\n\x13\x43\x65lExpressionConfig\x12 \n\x18\x65nable_string_conversion\x18\x01 \x01(\x08\x12\x1c\n\x14\x65nable_string_concat\x18\x02 \x01(\x08\x12\x1f\n\x17\x65nable_string_functions\x18\x03 \x01(\x08\x42|\n\"io.envoyproxy.envoy.config.core.v3B\x08\x43\x65lProtoP\x01ZBgithub.com/envoyproxy/go-control-plane/envoy/config/core/v3;corev3\xba\x80\xc8\xd1\x06\x02\x10\x02\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.config.core.v3.cel_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\"io.envoyproxy.envoy.config.core.v3B\010CelProtoP\001ZBgithub.com/envoyproxy/go-control-plane/envoy/config/core/v3;corev3\272\200\310\321\006\002\020\002'
  _globals['_CELEXPRESSIONCONFIG']._serialized_start=87
  _globals['_CELEXPRESSIONCONFIG']._serialized_end=205
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: google/api/cloudquotas/v1/resources.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'google/api/cloudquotas/v1/resources.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.api import field_behavior_pb2 as google_dot_api_dot_field__behavior__pb2
from google.api import resource_pb2 as google_dot_api_dot_resource__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n)google/api/cloudquotas/v1/resources.proto\x12\x19google.api.cloudquotas.v1\x1a\x1fgoogle/api/field_behavior.proto\x1a\x19google/api/resource.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x1egoogle/protobuf/wrappers.proto\"\x9f\x07\n\tQuotaInfo\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x10\n\x08quota_id\x18\x02 \x01(\t\x12\x0e\n\x06metric\x18\x03 \x01(\t\x12\x0f\n\x07service\x18\x04 \x01(\t\x12\x12\n\nis_precise\x18\x05 \x01(\x08\x12\x18\n\x10refresh_interval\x18\x06 \x01(\t\x12J\n\x0e\x63ontainer_type\x18\x07 \x01(\x0e\x32\x32.google.api.cloudquotas.v1.QuotaInfo.ContainerType\x12\x12\n\ndimensions\x18\x08 \x03(\t\x12\x1b\n\x13metric_display_name\x18\t \x01(\t\x12\x1a\n\x12quota_display_name\x18\n \x01(\t\x12\x13\n\x0bmetric_unit\x18\x0b \x01(\t\x12W\n\x1aquota_increase_eligibility\x18\x0c \x01(\x0b\x32\x33.google.api.cloudquotas.v1.QuotaIncreaseEligibility\x12\x10\n\x08is_fixed\x18\r \x01(\x08\x12\x43\n\x10\x64imensions_infos\x18\x0e \x03(\x0b\x32).google.api.cloudquotas.v1.DimensionsInfo\x12\x15\n\ris_concurrent\x18\x0f \x01(\x08\x12!\n\x19service_request_quota_uri\x18\x11 \x01(\t\"Z\n\rContainerType\x12\x1e\n\x1a\x43ONTAINER_TYPE_UNSPECIFIED\x10\x00\x12\x0b\n\x07PROJECT\x10\x01\x12\n\n\x06\x46OLDER\x10\x02\x12\x10\n\x0cORGANIZATION\x10\x03:\xae\x02\xea\x41\xaa\x02\n$cloudquotas.googleapis.com/QuotaInfo\x12Rprojects/{project}/locations/{location}/services/{service}/quotaInfos/{quota_info}\x12Pfolders/{folder}/locations/{location}/services/{service}/quotaInfos/{quota_info}\x12\\organizations/{organization}/locations/{location}/services/{service}/quotaInfos/{quota_info}\"\xae\x02\n\x18QuotaIncreaseEligibility\x12\x13\n\x0bis_eligible\x18\x01 \x01(\x08\x12\x65\n\x14ineligibility_reason\x18\x02 \x01(\x0e\x32G.google.api.cloudquotas.v1.QuotaIncreaseEligibility.IneligibilityReason\"\x95\x01\n\x13IneligibilityReason\x12$\n INELIGIBILITY_REASON_UNSPECIFIED\x10\x00\x12\x1c\n\x18NO_VALID_BILLING_ACCOUNT\x10\x01\x12\x11\n\rNOT_SUPPORTED\x10\x03\x12\x1c\n\x18NOT_ENOUGH_USAGE_HISTORY\x10\x04\x12\t\n\x05OTHER\x10\x02\"\x85\x06\n\x0fQuotaPreference\x12\x0c\n\x04name\x18\x01 \x01(\t\x12S\n\ndimensions\x18\x02 \x03(\x0b\x32:.google.api.cloudquotas.v1.QuotaPreference.DimensionsEntryB\x03\xe0\x41\x05\x12\x41\n\x0cquota_config\x18\x03 \x01(\x0b\x32&.google.api.cloudquotas.v1.QuotaConfigB\x03\xe0\x41\x02\x12\x11\n\x04\x65tag\x18\x04 \x01(\tB\x03\xe0\x41\x01\x12\x34\n\x0b\x63reate_time\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.TimestampB\x03\xe0\x41\x03\x12\x34\n\x0bupdate_time\x18\x06 \x01(\x0b\x32\x1a.google.protobuf.TimestampB\x03\xe0\x41\x03\x12\x14\n\x07service\x18\x07 \x01(\tB\x03\xe0\x41\x02\x12\x15\n\x08quota_id\x18\x08 \x01(\tB\x03\xe0\x41\x02\x12\x18\n\x0breconciling\x18\n \x01(\x08\x42\x03\xe0\x41\x03\x12\x15\n\rjustification\x18\x0b \x01(\t\x12\x1a\n\rcontact_email\x18\x0c \x01(\tB\x03\xe0\x41\x04\x1a\x31\n\x0f\x44imensionsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01:\x9f\x02\xea\x41\x9b\x02\n*cloudquotas.googleapis.com/QuotaPreference\x12Kprojects/{project}/locations/{location}/quotaPreferences/{quota_preference}\x12Ifolders/{folder}/locations/{location}/quotaPreferences/{quota_preference}\x12Uorganizations/{organization}/locations/{location}/quotaPreferences/{quota_preference}\"\xb1\x03\n\x0bQuotaConfig\x12\x1c\n\x0fpreferred_value\x18\x01 \x01(\x03\x42\x03\xe0\x41\x02\x12\x19\n\x0cstate_detail\x18\x02 \x01(\tB\x03\xe0\x41\x03\x12\x37\n\rgranted_value\x18\x03 \x01(\x0b\x32\x1b.google.protobuf.Int64ValueB\x03\xe0\x41\x03\x12\x15\n\x08trace_id\x18\x04 \x01(\tB\x03\xe0\x41\x03\x12Q\n\x0b\x61nnotations\x18\x05 \x03(\x0b\x32\x37.google.api.cloudquotas.v1.QuotaConfig.AnnotationsEntryB\x03\xe0\x41\x01\x12J\n\x0erequest_origin\x18\x06 \x01(\x0e\x32-.google.api.cloudquotas.v1.QuotaConfig.OriginB\x03\xe0\x41\x03\x1a\x32\n\x10\x41nnotationsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"F\n\x06Origin\x12\x16\n\x12ORIGIN_UNSPECIFIED\x10\x00\x12\x11\n\rCLOUD_CONSOLE\x10\x01\x12\x11\n\rAUTO_ADJUSTER\x10\x02\"\xea\x01\n\x0e\x44imensionsInfo\x12M\n\ndimensions\x18\x01 \x03(\x0b\x32\x39.google.api.cloudquotas.v1.DimensionsInfo.DimensionsEntry\x12\x38\n\x07\x64\x65tails\x18\x02 \x01(\x0b\x32\'.google.api.cloudquotas.v1.QuotaDetails\x12\x1c\n\x14\x61pplicable_locations\x18\x03 \x03(\t\x1a\x31\n\x0f\x44imensionsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"[\n\x0cQuotaDetails\x12\r\n\x05value\x18\x01 \x01(\x03\x12<\n\x0crollout_info\x18\x03 \x01(\x0b\x32&.google.api.cloudquotas.v1.RolloutInfo\"&\n\x0bRolloutInfo\x12\x17\n\x0fongoing_rollout\x18\x01 \x01(\x08*~\n\x10QuotaSafetyCheck\x12\"\n\x1eQUOTA_SAFETY_CHECK_UNSPECIFIED\x10\x00\x12\x1e\n\x1aQUOTA_DECREASE_BELOW_USAGE\x10\x01\x12&\n\"QUOTA_DECREASE_PERCENTAGE_TOO_HIGH\x10\x02\x42\xd1\x01\n\x1d\x63om.google.api.cloudquotas.v1B\x0eResourcesProtoP\x01ZAcloud.google.com/go/cloudquotas/apiv1/cloudquotaspb;cloudquotaspb\xaa\x02\x1bGoogle.Cloud.CloudQuotas.V1\xca\x02\x1bGoogle\\Cloud\\CloudQuotas\\V1\xea\x02\x1eGoogle::Cloud::CloudQuotas::V1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'google.api.cloudquotas.v1.resources_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\035com.google.api.cloudquotas.v1B\016ResourcesProtoP\001ZAcloud.google.com/go/cloudquotas/apiv1/cloudquotaspb;cloudquotaspb\252\002\033Google.Cloud.CloudQuotas.V1\312\002\033Google\\Cloud\\CloudQuotas\\V1\352\002\036Google::Cloud::CloudQuotas::V1'
  _globals['_QUOTAINFO']._loaded_options = None
  _globals['_QUOTAINFO']._serialized_options = b'\352A\252\002\n$cloudquotas.googleapis.com/QuotaInfo\022Rprojects/{project}/locations/{location}/services/{service}/quotaInfos/{quota_info}\022Pfolders/{folder}/locations/{location}/services/{service}/quotaInfos/{quota_info}\022\\organizations/{organization}/locations/{location}/services/{service}/quotaInfos/{quota_info}'
  _globals['_QUOTAPREFERENCE_DIMENSIONSENTRY']._loaded_options = None
  _globals['_QUOTAPREFERENCE_DIMENSIONSENTRY']._serialized_options = b'8\001'
  _globals['_QUOTAPREFERENCE'].fields_by_name['dimensions']._loaded_options = None
  _globals['_QUOTAPREFERENCE'].fields_by_name['dimensions']._serialized_options = b'\340A\005'
  _globals['_QUOTAPREFERENCE'].fields_by_name['quota_config']._loaded_options = None
  _globals['_QUOTAPREFERENCE'].fields_by_name['quota_config']._serialized_options = b'\340A\002'
  _globals['_QUOTAPREFERENCE'].fields_by_name['etag']._loaded_options = None
  _globals['_QUOTAPREFERENCE'].fields_by_name['etag']._serialized_options = b'\340A\001'
  _globals['_QUOTAPREFERENCE'].fields_by_name['create_time']._loaded_options = None
  _globals['_QUOTAPREFERENCE'].fields_by_name['create_time']._serialized_options = b'\340A\003'
  _globals['_QUOTAPREFERENCE'].fields_by_name['update_time']._loaded_options = None
  _globals['_QUOTAPREFERENCE'].fields_by_name['update_time']._serialized_options = b'\340A\003'
  _globals['_QUOTAPREFERENCE'].fields_by_name['service']._loaded_options = None
  _globals['_QUOTAPREFERENCE'].fields_by_name['service']._serialized_options = b'\340A\002'
  _globals['_QUOTAPREFERENCE'].fields_by_name['quota_id']._loaded_options = None
  _globals['_QUOTAPREFERENCE'].fields_by_name['quota_id']._serialized_options = b'\340A\002'
  _globals['_QUOTAPREFERENCE'].fields_by_name['reconciling']._loaded_options = None
  _globals['_QUOTAPREFERENCE'].fields_by_name['reconciling']._serialized_options = b'\340A\003'
  _globals['_QUOTAPREFERENCE'].fields_by_name['contact_email']._loaded_options = None
  _globals['_QUOTAPREFERENCE'].fields_by_name['contact_email']._serialized_options = b'\340A\004'
  _globals['_QUOTAPREFERENCE']._loaded_options = None
  _globals['_QUOTAPREFERENCE']._serialized_options = b'\352A\233\002\n*cloudquotas.googleapis.com/QuotaPreference\022Kprojects/{project}/locations/{location}/quotaPreferences/{quota_preference}\022Ifolders/{folder}/locations/{location}/quotaPreferences/{quota_preference}\022Uorganizations/{organization}/locations/{location}/quotaPreferences/{quota_preference}'
  _globals['_QUOTACONFIG_ANNOTATIONSENTRY']._loaded_options = None
  _globals['_QUOTACONFIG_ANNOTATIONSENTRY']._serialized_options = b'8\001'
  _globals['_QUOTACONFIG'].fields_by_name['preferred_value']._loaded_options = None
  _globals['_QUOTACONFIG'].fields_by_name['preferred_value']._serialized_options = b'\340A\002'
  _globals['_QUOTACONFIG'].fields_by_name['state_detail']._loaded_options = None
  _globals['_QUOTACONFIG'].fields_by_name['state_detail']._serialized_options = b'\340A\003'
  _globals['_QUOTACONFIG'].fields_by_name['granted_value']._loaded_options = None
  _globals['_QUOTACONFIG'].fields_by_name['granted_value']._serialized_options = b'\340A\003'
  _globals['_QUOTACONFIG'].fields_by_name['trace_id']._loaded_options = None
  _globals['_QUOTACONFIG'].fields_by_name['trace_id']._serialized_options = b'\340A\003'
  _globals['_QUOTACONFIG'].fields_by_name['annotations']._loaded_options = None
  _globals['_QUOTACONFIG'].fields_by_name['annotations']._serialized_options = b'\340A\001'
  _globals['_QUOTACONFIG'].fields_by_name['request_origin']._loaded_options = None
  _globals['_QUOTACONFIG'].fields_by_name['request_origin']._serialized_options = b'\340A\003'
  _globals['_DIMENSIONSINFO_DIMENSIONSENTRY']._loaded_options = None
  _globals['_DIMENSIONSINFO_DIMENSIONSENTRY']._serialized_options = b'8\001'
  _globals['_QUOTASAFETYCHECK']._serialized_start=3014
  _globals['_QUOTASAFETYCHECK']._serialized_end=3140
  _globals['_QUOTAINFO']._serialized_start=198
  _globals['_QUOTAINFO']._serialized_end=1125
  _globals['_QUOTAINFO_CONTAINERTYPE']._serialized_start=730
  _globals['_QUOTAINFO_CONTAINERTYPE']._serialized_end=820
  _globals['_QUOTAINCREASEELIGIBILITY']._serialized_start=1128
  _globals['_QUOTAINCREASEELIGIBILITY']._serialized_end=1430
  _globals['_QUOTAINCREASEELIGIBILITY_INELIGIBILITYREASON']._serialized_start=1281
  _globals['_QUOTAINCREASEELIGIBILITY_INELIGIBILITYREASON']._serialized_end=1430
  _globals['_QUOTAPREFERENCE']._serialized_start=1433
  _globals['_QUOTAPREFERENCE']._serialized_end=2206
  _globals['_QUOTAPREFERENCE_DIMENSIONSENTRY']._serialized_start=1867
  _globals['_QUOTAPREFERENCE_DIMENSIONSENTRY']._serialized_end=1916
  _globals['_QUOTACONFIG']._serialized_start=2209
  _globals['_QUOTACONFIG']._serialized_end=2642
  _globals['_QUOTACONFIG_ANNOTATIONSENTRY']._serialized_start=2520
  _globals['_QUOTACONFIG_ANNOTATIONSENTRY']._serialized_end=2570
  _globals['_QUOTACONFIG_ORIGIN']._serialized_start=2572
  _globals['_QUOTACONFIG_ORIGIN']._serialized_end=2642
  _globals['_DIMENSIONSINFO']._serialized_start=2645
  _globals['_DIMENSIONSINFO']._serialized_end=2879
  _globals['_DIMENSIONSINFO_DIMENSIONSENTRY']._serialized_start=1867
  _globals['_DIMENSIONSINFO_DIMENSIONSENTRY']._serialized_end=1916
  _globals['_QUOTADETAILS']._serialized_start=2881
  _globals['_QUOTADETAILS']._serialized_end=2972
  _globals['_ROLLOUTINFO']._serialized_start=2974
  _globals['_ROLLOUTINFO']._serialized_end=3012
# @@protoc_insertion_point(module_scope)



# Copyright 2015 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Generate XML and HTML test reports."""

try:
    from mako import exceptions
    from mako.runtime import Context
    from mako.template import Template
except ImportError:
    pass  # Mako not installed but it is ok.
import datetime
import os
import string
import xml.etree.cElementTree as ET


def _filter_msg(msg, output_format):
    """Filters out nonprintable and illegal characters from the message."""
    if output_format in ["XML", "HTML"]:
        if isinstance(msg, bytes):
            decoded_msg = msg.decode("UTF-8", "ignore")
        else:
            decoded_msg = msg
        # keep whitespaces but remove formfeed and vertical tab characters
        # that make XML report unparsable.
        filtered_msg = "".join(
            filter(
                lambda x: x in string.printable and x != "\f" and x != "\v",
                decoded_msg,
            )
        )
        if output_format == "HTML":
            filtered_msg = filtered_msg.replace('"', "&quot;")
        return filtered_msg
    else:
        return msg


def new_junit_xml_tree():
    return ET.ElementTree(ET.Element("testsuites"))


def render_junit_xml_report(
    resultset,
    report_file,
    suite_package="grpc",
    suite_name="tests",
    replace_dots=True,
    multi_target=False,
):
    """Generate JUnit-like XML report."""
    if not multi_target:
        tree = new_junit_xml_tree()
        append_junit_xml_results(
            tree, resultset, suite_package, suite_name, "1", replace_dots
        )
        create_xml_report_file(tree, report_file)
    else:
        # To have each test result displayed as a separate target by the Resultstore/Sponge UI,
        # we generate a separate XML report file for each test result
        for shortname, results in resultset.items():
            one_result = {shortname: results}
            tree = new_junit_xml_tree()
            append_junit_xml_results(
                tree,
                one_result,
                "%s_%s" % (suite_package, shortname),
                "%s_%s" % (suite_name, shortname),
                "1",
                replace_dots,
            )
            per_suite_report_file = os.path.join(
                os.path.dirname(report_file),
                shortname,
                os.path.basename(report_file),
            )
            create_xml_report_file(tree, per_suite_report_file)


def create_xml_report_file(tree, report_file):
    """Generate JUnit-like report file from xml tree ."""
    # env variable can be used to override the base location for the reports
    base_dir = os.getenv("GRPC_TEST_REPORT_BASE_DIR", None)
    if base_dir:
        report_file = os.path.join(base_dir, report_file)
    # ensure the report directory exists
    report_dir = os.path.dirname(os.path.abspath(report_file))
    if not os.path.exists(report_dir):
        os.makedirs(report_dir)
    tree.write(report_file, encoding="UTF-8")


def append_junit_xml_results(
    tree, resultset, suite_package, suite_name, id, replace_dots=True
):
    """Append a JUnit-like XML report tree with test results as a new suite."""
    if replace_dots:
        # ResultStore UI displays test suite names containing dots only as the component
        # after the last dot, which results bad info being displayed in the UI.
        # We replace dots by another character to avoid this problem.
        suite_name = suite_name.replace(".", "_")
    testsuite = ET.SubElement(
        tree.getroot(),
        "testsuite",
        id=id,
        package=suite_package,
        name=suite_name,
        timestamp=datetime.datetime.now().isoformat(),
    )
    failure_count = 0
    error_count = 0
    for shortname, results in resultset.items():
        for result in results:
            xml_test = ET.SubElement(testsuite, "testcase", name=shortname)
            if result.elapsed_time:
                xml_test.set("time", str(result.elapsed_time))
            filtered_msg = _filter_msg(result.message, "XML")
            if result.state == "FAILED":
                ET.SubElement(xml_test, "failure", message="Failure").text = (
                    filtered_msg
                )
                failure_count += 1
            elif result.state == "TIMEOUT":
                ET.SubElement(xml_test, "error", message="Timeout").text = (
                    filtered_msg
                )
                error_count += 1
            elif result.state == "SKIPPED":
                ET.SubElement(xml_test, "skipped", message="Skipped")
    testsuite.set("failures", str(failure_count))
    testsuite.set("errors", str(error_count))


def render_interop_html_report(
    client_langs,
    server_langs,
    test_cases,
    auth_test_cases,
    http2_cases,
    http2_server_cases,
    resultset,
    num_failures,
    cloud_to_prod,
    prod_servers,
    http2_interop,
):
    """Generate HTML report for interop tests."""
    template_file = "tools/run_tests/interop/interop_html_report.template"
    try:
        mytemplate = Template(filename=template_file, format_exceptions=True)
    except NameError:
        print(
            "Mako template is not installed. Skipping HTML report generation."
        )
        return
    except IOError as e:
        print(("Failed to find the template %s: %s" % (template_file, e)))
        return

    sorted_test_cases = sorted(test_cases)
    sorted_auth_test_cases = sorted(auth_test_cases)
    sorted_http2_cases = sorted(http2_cases)
    sorted_http2_server_cases = sorted(http2_server_cases)
    sorted_client_langs = sorted(client_langs)
    sorted_server_langs = sorted(server_langs)
    sorted_prod_servers = sorted(prod_servers)

    args = {
        "client_langs": sorted_client_langs,
        "server_langs": sorted_server_langs,
        "test_cases": sorted_test_cases,
        "auth_test_cases": sorted_auth_test_cases,
        "http2_cases": sorted_http2_cases,
        "http2_server_cases": sorted_http2_server_cases,
        "resultset": resultset,
        "num_failures": num_failures,
        "cloud_to_prod": cloud_to_prod,
        "prod_servers": sorted_prod_servers,
        "http2_interop": http2_interop,
    }

    html_report_out_dir = "reports"
    if not os.path.exists(html_report_out_dir):
        os.mkdir(html_report_out_dir)
    html_file_path = os.path.join(html_report_out_dir, "index.html")
    try:
        with open(html_file_path, "w") as output_file:
            mytemplate.render_context(Context(output_file, **args))
    except:
        print((exceptions.text_error_template().render()))
        raise


def render_perf_profiling_results(output_filepath, profile_names):
    with open(output_filepath, "w") as output_file:
        output_file.write("<ul>\n")
        for name in profile_names:
            output_file.write("<li><a href=%s>%s</a></li>\n" % (name, name))
        output_file.write("</ul>\n")



# Copyright 2020 The gRPC Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Poll statistics from the server."""

import argparse
import asyncio
import logging

import grpc
from grpc_channelz.v1 import channelz_pb2
from grpc_channelz.v1 import channelz_pb2_grpc


async def run(addr: str) -> None:
    async with grpc.aio.insecure_channel(addr) as channel:
        channelz_stub = channelz_pb2_grpc.ChannelzStub(channel)
        response = await channelz_stub.GetServers(
            channelz_pb2.GetServersRequest(start_server_id=0)
        )
        print("Info for all servers: %s" % response)


async def main() -> None:
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--addr",
        nargs=1,
        type=str,
        default="[::]:50051",
        help="the address to request",
    )
    args = parser.parse_args()
    run(addr=args.addr)


if __name__ == "__main__":
    logging.basicConfig()
    asyncio.get_event_loop().run_until_complete(main())



# Copyright 2020 The gRPC Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import asyncio
import datetime
import logging
import unittest

import grpc
from grpc.experimental import aio

from src.proto.grpc.testing import messages_pb2
from src.proto.grpc.testing import test_pb2_grpc
from tests.unit.framework.common import test_constants
from tests_aio.unit._common import CountingResponseIterator
from tests_aio.unit._common import inject_callbacks
from tests_aio.unit._constants import UNREACHABLE_TARGET
from tests_aio.unit._test_base import AioTestBase
from tests_aio.unit._test_server import start_test_server

_SHORT_TIMEOUT_S = 1.0

_NUM_STREAM_RESPONSES = 5
_REQUEST_PAYLOAD_SIZE = 7
_RESPONSE_PAYLOAD_SIZE = 7
_RESPONSE_INTERVAL_US = int(_SHORT_TIMEOUT_S * 1000 * 1000)


class _UnaryStreamInterceptorEmpty(aio.UnaryStreamClientInterceptor):
    async def intercept_unary_stream(
        self, continuation, client_call_details, request
    ):
        return await continuation(client_call_details, request)

    def assert_in_final_state(self, test: unittest.TestCase):
        pass


class _UnaryStreamInterceptorWithResponseIterator(
    aio.UnaryStreamClientInterceptor
):
    async def intercept_unary_stream(
        self, continuation, client_call_details, request
    ):
        call = await continuation(client_call_details, request)
        self.response_iterator = CountingResponseIterator(call)
        return self.response_iterator

    def assert_in_final_state(self, test: unittest.TestCase):
        test.assertEqual(
            _NUM_STREAM_RESPONSES, self.response_iterator.response_cnt
        )


class TestUnaryStreamClientInterceptor(AioTestBase):
    async def setUp(self):
        self._server_target, self._server = await start_test_server()

    async def tearDown(self):
        await self._server.stop(None)

    async def test_intercepts(self):
        for interceptor_class in (
            _UnaryStreamInterceptorEmpty,
            _UnaryStreamInterceptorWithResponseIterator,
        ):
            with self.subTest(name=interceptor_class):
                interceptor = interceptor_class()

                request = messages_pb2.StreamingOutputCallRequest()
                request.response_parameters.extend(
                    [
                        messages_pb2.ResponseParameters(
                            size=_RESPONSE_PAYLOAD_SIZE
                        )
                    ]
                    * _NUM_STREAM_RESPONSES
                )

                channel = aio.insecure_channel(
                    self._server_target, interceptors=[interceptor]
                )
                stub = test_pb2_grpc.TestServiceStub(channel)
                call = stub.StreamingOutputCall(request)

                await call.wait_for_connection()

                response_cnt = 0
                async for response in call:
                    response_cnt += 1
                    self.assertIs(
                        type(response), messages_pb2.StreamingOutputCallResponse
                    )
                    self.assertEqual(
                        _RESPONSE_PAYLOAD_SIZE, len(response.payload.body)
                    )

                self.assertEqual(response_cnt, _NUM_STREAM_RESPONSES)
                self.assertEqual(await call.code(), grpc.StatusCode.OK)
                self.assertEqual(await call.initial_metadata(), aio.Metadata())
                self.assertEqual(await call.trailing_metadata(), aio.Metadata())
                self.assertEqual(await call.details(), "")
                self.assertEqual(await call.debug_error_string(), "")
                self.assertEqual(call.cancel(), False)
                self.assertEqual(call.cancelled(), False)
                self.assertEqual(call.done(), True)

                interceptor.assert_in_final_state(self)

                await channel.close()

    async def test_add_done_callback_interceptor_task_not_finished(self):
        for interceptor_class in (
            _UnaryStreamInterceptorEmpty,
            _UnaryStreamInterceptorWithResponseIterator,
        ):
            with self.subTest(name=interceptor_class):
                interceptor = interceptor_class()

                request = messages_pb2.StreamingOutputCallRequest()
                request.response_parameters.extend(
                    [
                        messages_pb2.ResponseParameters(
                            size=_RESPONSE_PAYLOAD_SIZE
                        )
                    ]
                    * _NUM_STREAM_RESPONSES
                )

                channel = aio.insecure_channel(
                    self._server_target, interceptors=[interceptor]
                )
                stub = test_pb2_grpc.TestServiceStub(channel)
                call = stub.StreamingOutputCall(request)

                validation = inject_callbacks(call)

                async for response in call:
                    pass

                await validation

                await channel.close()

    async def test_add_done_callback_interceptor_task_finished(self):
        for interceptor_class in (
            _UnaryStreamInterceptorEmpty,
            _UnaryStreamInterceptorWithResponseIterator,
        ):
            with self.subTest(name=interceptor_class):
                interceptor = interceptor_class()

                request = messages_pb2.StreamingOutputCallRequest()
                request.response_parameters.extend(
                    [
                        messages_pb2.ResponseParameters(
                            size=_RESPONSE_PAYLOAD_SIZE
                        )
                    ]
                    * _NUM_STREAM_RESPONSES
                )

                channel = aio.insecure_channel(
                    self._server_target, interceptors=[interceptor]
                )
                stub = test_pb2_grpc.TestServiceStub(channel)
                call = stub.StreamingOutputCall(request)

                # This ensures that the callbacks will be registered
                # with the intercepted call rather than saving in the
                # pending state list.
                await call.wait_for_connection()

                validation = inject_callbacks(call)

                async for response in call:
                    pass

                await validation

                await channel.close()

    async def test_response_iterator_using_read(self):
        interceptor = _UnaryStreamInterceptorWithResponseIterator()

        channel = aio.insecure_channel(
            self._server_target, interceptors=[interceptor]
        )
        stub = test_pb2_grpc.TestServiceStub(channel)

        request = messages_pb2.StreamingOutputCallRequest()
        request.response_parameters.extend(
            [messages_pb2.ResponseParameters(size=_RESPONSE_PAYLOAD_SIZE)]
            * _NUM_STREAM_RESPONSES
        )

        call = stub.StreamingOutputCall(request)

        response_cnt = 0
        for response in range(_NUM_STREAM_RESPONSES):
            response = await call.read()
            response_cnt += 1
            self.assertIs(
                type(response), messages_pb2.StreamingOutputCallResponse
            )
            self.assertEqual(_RESPONSE_PAYLOAD_SIZE, len(response.payload.body))

        self.assertEqual(response_cnt, _NUM_STREAM_RESPONSES)
        self.assertEqual(
            interceptor.response_iterator.response_cnt, _NUM_STREAM_RESPONSES
        )
        self.assertEqual(await call.code(), grpc.StatusCode.OK)

        await channel.close()

    async def test_too_many_reads(self):
        for interceptor_class in (
            [_UnaryStreamInterceptorEmpty],
            [_UnaryStreamInterceptorWithResponseIterator],
            [],
        ):
            with self.subTest(name=interceptor_class):
                if interceptor_class:
                    interceptor = interceptor_class[0]()
                    channel = aio.insecure_channel(
                        self._server_target, interceptors=[interceptor]
                    )
                else:
                    channel = aio.insecure_channel(self._server_target)
                stub = test_pb2_grpc.TestServiceStub(channel)

                request = messages_pb2.StreamingOutputCallRequest()
                request.response_parameters.extend(
                    [
                        messages_pb2.ResponseParameters(
                            size=_RESPONSE_PAYLOAD_SIZE
                        )
                    ]
                    * _NUM_STREAM_RESPONSES
                )

                call = stub.StreamingOutputCall(request)

                response_cnt = 0
                for response in range(_NUM_STREAM_RESPONSES):
                    response = await call.read()
                    response_cnt += 1
                    self.assertIs(
                        type(response), messages_pb2.StreamingOutputCallResponse
                    )
                    self.assertEqual(
                        _RESPONSE_PAYLOAD_SIZE, len(response.payload.body)
                    )

                # Additional read() should return EOF
                self.assertIs(await call.read(), aio.EOF)

                self.assertEqual(await call.code(), grpc.StatusCode.OK)
                # After the RPC finished, the read should also produce EOF
                self.assertIs(await call.read(), aio.EOF)

                await channel.close()

    async def test_multiple_interceptors_response_iterator(self):
        for interceptor_class in (
            _UnaryStreamInterceptorEmpty,
            _UnaryStreamInterceptorWithResponseIterator,
        ):
            with self.subTest(name=interceptor_class):
                interceptors = [interceptor_class(), interceptor_class()]

                channel = aio.insecure_channel(
                    self._server_target, interceptors=interceptors
                )
                stub = test_pb2_grpc.TestServiceStub(channel)

                request = messages_pb2.StreamingOutputCallRequest()
                request.response_parameters.extend(
                    [
                        messages_pb2.ResponseParameters(
                            size=_RESPONSE_PAYLOAD_SIZE
                        )
                    ]
                    * _NUM_STREAM_RESPONSES
                )

                call = stub.StreamingOutputCall(request)

                response_cnt = 0
                async for response in call:
                    response_cnt += 1
                    self.assertIs(
                        type(response), messages_pb2.StreamingOutputCallResponse
                    )
                    self.assertEqual(
                        _RESPONSE_PAYLOAD_SIZE, len(response.payload.body)
                    )

                self.assertEqual(response_cnt, _NUM_STREAM_RESPONSES)
                self.assertEqual(await call.code(), grpc.StatusCode.OK)

                await channel.close()

    async def test_intercepts_response_iterator_rpc_error(self):
        for interceptor_class in (
            _UnaryStreamInterceptorEmpty,
            _UnaryStreamInterceptorWithResponseIterator,
        ):
            with self.subTest(name=interceptor_class):
                channel = aio.insecure_channel(
                    UNREACHABLE_TARGET, interceptors=[interceptor_class()]
                )
                request = messages_pb2.StreamingOutputCallRequest()
                stub = test_pb2_grpc.TestServiceStub(channel)
                call = stub.StreamingOutputCall(request)

                with self.assertRaises(aio.AioRpcError) as exception_context:
                    async for response in call:
                        pass

                self.assertEqual(
                    grpc.StatusCode.UNAVAILABLE,
                    exception_context.exception.code(),
                )

                self.assertTrue(call.done())
                self.assertEqual(grpc.StatusCode.UNAVAILABLE, await call.code())
                await channel.close()

    async def test_cancel_before_rpc(self):
        interceptor_reached = asyncio.Event()
        wait_for_ever = self.loop.create_future()

        class Interceptor(aio.UnaryStreamClientInterceptor):
            async def intercept_unary_stream(
                self, continuation, client_call_details, request
            ):
                interceptor_reached.set()
                await wait_for_ever

        channel = aio.insecure_channel(
            UNREACHABLE_TARGET, interceptors=[Interceptor()]
        )
        request = messages_pb2.StreamingOutputCallRequest()
        stub = test_pb2_grpc.TestServiceStub(channel)
        call = stub.StreamingOutputCall(request)

        self.assertFalse(call.cancelled())
        self.assertFalse(call.done())

        await interceptor_reached.wait()
        self.assertTrue(call.cancel())

        with self.assertRaises(asyncio.CancelledError):
            async for response in call:
                pass

        self.assertTrue(call.cancelled())
        self.assertTrue(call.done())
        self.assertEqual(await call.code(), grpc.StatusCode.CANCELLED)
        self.assertEqual(await call.initial_metadata(), None)
        self.assertEqual(await call.trailing_metadata(), None)
        await channel.close()

    async def test_cancel_after_rpc(self):
        interceptor_reached = asyncio.Event()
        wait_for_ever = self.loop.create_future()

        class Interceptor(aio.UnaryStreamClientInterceptor):
            async def intercept_unary_stream(
                self, continuation, client_call_details, request
            ):
                call = await continuation(client_call_details, request)
                interceptor_reached.set()
                await wait_for_ever

        channel = aio.insecure_channel(
            UNREACHABLE_TARGET, interceptors=[Interceptor()]
        )
        request = messages_pb2.StreamingOutputCallRequest()
        stub = test_pb2_grpc.TestServiceStub(channel)
        call = stub.StreamingOutputCall(request)

        self.assertFalse(call.cancelled())
        self.assertFalse(call.done())

        await interceptor_reached.wait()
        self.assertTrue(call.cancel())

        with self.assertRaises(asyncio.CancelledError):
            async for response in call:
                pass

        self.assertTrue(call.cancelled())
        self.assertTrue(call.done())
        self.assertEqual(await call.code(), grpc.StatusCode.CANCELLED)
        self.assertEqual(await call.initial_metadata(), None)
        self.assertEqual(await call.trailing_metadata(), None)
        await channel.close()

    async def test_cancel_consuming_response_iterator(self):
        request = messages_pb2.StreamingOutputCallRequest()
        request.response_parameters.extend(
            [messages_pb2.ResponseParameters(size=_RESPONSE_PAYLOAD_SIZE)]
            * _NUM_STREAM_RESPONSES
        )

        channel = aio.insecure_channel(
            self._server_target,
            interceptors=[_UnaryStreamInterceptorWithResponseIterator()],
        )
        stub = test_pb2_grpc.TestServiceStub(channel)
        call = stub.StreamingOutputCall(request)

        with self.assertRaises(asyncio.CancelledError):
            async for response in call:
                call.cancel()

        self.assertTrue(call.cancelled())
        self.assertTrue(call.done())
        self.assertEqual(await call.code(), grpc.StatusCode.CANCELLED)
        await channel.close()

    async def test_cancel_by_the_interceptor(self):
        class Interceptor(aio.UnaryStreamClientInterceptor):
            async def intercept_unary_stream(
                self, continuation, client_call_details, request
            ):
                call = await continuation(client_call_details, request)
                call.cancel()
                return call

        channel = aio.insecure_channel(
            UNREACHABLE_TARGET, interceptors=[Interceptor()]
        )
        request = messages_pb2.StreamingOutputCallRequest()
        stub = test_pb2_grpc.TestServiceStub(channel)
        call = stub.StreamingOutputCall(request)

        with self.assertRaises(asyncio.CancelledError):
            async for response in call:
                pass

        self.assertTrue(call.cancelled())
        self.assertTrue(call.done())
        self.assertEqual(await call.code(), grpc.StatusCode.CANCELLED)
        await channel.close()

    async def test_exception_raised_by_interceptor(self):
        class InterceptorException(Exception):
            pass

        class Interceptor(aio.UnaryStreamClientInterceptor):
            async def intercept_unary_stream(
                self, continuation, client_call_details, request
            ):
                raise InterceptorException

        channel = aio.insecure_channel(
            UNREACHABLE_TARGET, interceptors=[Interceptor()]
        )
        request = messages_pb2.StreamingOutputCallRequest()
        stub = test_pb2_grpc.TestServiceStub(channel)
        call = stub.StreamingOutputCall(request)

        with self.assertRaises(InterceptorException):
            async for response in call:
                pass

        await channel.close()


if __name__ == "__main__":
    logging.basicConfig(level=logging.DEBUG)
    unittest.main(verbosity=2)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: helloworld.proto
# Protobuf Python Version: 5.28.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    28,
    1,
    '',
    'helloworld.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x10helloworld.proto\x12\nhelloworld\"\x1c\n\x0cHelloRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\"\x1d\n\nHelloReply\x12\x0f\n\x07message\x18\x01 \x01(\t2\xe4\x01\n\x07Greeter\x12>\n\x08SayHello\x12\x18.helloworld.HelloRequest\x1a\x16.helloworld.HelloReply\"\x00\x12K\n\x13SayHelloStreamReply\x12\x18.helloworld.HelloRequest\x1a\x16.helloworld.HelloReply\"\x00\x30\x01\x12L\n\x12SayHelloBidiStream\x12\x18.helloworld.HelloRequest\x1a\x16.helloworld.HelloReply\"\x00(\x01\x30\x01\x42\x36\n\x1bio.grpc.examples.helloworldB\x0fHelloWorldProtoP\x01\xa2\x02\x03HLWb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'helloworld_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\033io.grpc.examples.helloworldB\017HelloWorldProtoP\001\242\002\003HLW'
  _globals['_HELLOREQUEST']._serialized_start=32
  _globals['_HELLOREQUEST']._serialized_end=60
  _globals['_HELLOREPLY']._serialized_start=62
  _globals['_HELLOREPLY']._serialized_end=91
  _globals['_GREETER']._serialized_start=94
  _globals['_GREETER']._serialized_end=322
# @@protoc_insertion_point(module_scope)



# Copyright 2018 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""The Python implementation of the GRPC interoperability test client."""

import argparse
import logging
import sys

from tests.fork import methods


def _args():
    def parse_bool(value):
        if value == "true":
            return True
        if value == "false":
            return False
        raise argparse.ArgumentTypeError("Only true/false allowed")

    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--server_host",
        default="localhost",
        type=str,
        help="the host to which to connect",
    )
    parser.add_argument(
        "--server_port",
        type=int,
        required=True,
        help="the port to which to connect",
    )
    parser.add_argument(
        "--test_case",
        default="large_unary",
        type=str,
        help="the test case to execute",
    )
    parser.add_argument(
        "--use_tls",
        default=False,
        type=parse_bool,
        help="require a secure connection",
    )
    return parser.parse_args()


def _test_case_from_arg(test_case_arg):
    for test_case in methods.TestCase:
        if test_case_arg == test_case.value:
            return test_case
    else:
        raise ValueError('No test case "%s"!' % test_case_arg)


def test_fork():
    logging.basicConfig(level=logging.INFO)
    args = vars(_args())
    if args["test_case"] == "all":
        for test_case in methods.TestCase:
            test_case.run_test(args)
    else:
        test_case = _test_case_from_arg(args["test_case"])
        test_case.run_test(args)


if __name__ == "__main__":
    test_fork()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/extensions/filters/network/thrift_proxy/filters/header_to_metadata/v3/header_to_metadata.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/extensions/filters/network/thrift_proxy/filters/header_to_metadata/v3/header_to_metadata.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from envoy.type.matcher.v3 import regex_pb2 as envoy_dot_type_dot_matcher_dot_v3_dot_regex__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\ndenvoy/extensions/filters/network/thrift_proxy/filters/header_to_metadata/v3/header_to_metadata.proto\x12Kenvoy.extensions.filters.network.thrift_proxy.filters.header_to_metadata.v3\x1a!envoy/type/matcher/v3/regex.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\xdb\x07\n\x10HeaderToMetadata\x12\x83\x01\n\rrequest_rules\x18\x01 \x03(\x0b\x32\x62.envoy.extensions.filters.network.thrift_proxy.filters.header_to_metadata.v3.HeaderToMetadata.RuleB\x08\xfa\x42\x05\x92\x01\x02\x08\x01\x1a\xaa\x03\n\x0cKeyValuePair\x12\x1a\n\x12metadata_namespace\x18\x01 \x01(\t\x12\x14\n\x03key\x18\x02 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\x12\x0f\n\x05value\x18\x03 \x01(\tH\x00\x12M\n\x13regex_value_rewrite\x18\x04 \x01(\x0b\x32..envoy.type.matcher.v3.RegexMatchAndSubstituteH\x00\x12\x7f\n\x04type\x18\x05 \x01(\x0e\x32g.envoy.extensions.filters.network.thrift_proxy.filters.header_to_metadata.v3.HeaderToMetadata.ValueTypeB\x08\xfa\x42\x05\x82\x01\x02\x10\x01\x12y\n\x06\x65ncode\x18\x06 \x01(\x0e\x32i.envoy.extensions.filters.network.thrift_proxy.filters.header_to_metadata.v3.HeaderToMetadata.ValueEncodeB\x0c\n\nvalue_type\x1a\xb5\x02\n\x04Rule\x12\x1d\n\x06header\x18\x01 \x01(\tB\r\xfa\x42\nr\x08\x10\x01\xc0\x01\x01\xc8\x01\x00\x12~\n\non_present\x18\x02 \x01(\x0b\x32j.envoy.extensions.filters.network.thrift_proxy.filters.header_to_metadata.v3.HeaderToMetadata.KeyValuePair\x12~\n\non_missing\x18\x03 \x01(\x0b\x32j.envoy.extensions.filters.network.thrift_proxy.filters.header_to_metadata.v3.HeaderToMetadata.KeyValuePair\x12\x0e\n\x06remove\x18\x04 \x01(\x08\"7\n\tValueType\x12\n\n\x06STRING\x10\x00\x12\n\n\x06NUMBER\x10\x01\x12\x12\n\x0ePROTOBUF_VALUE\x10\x02\"#\n\x0bValueEncode\x12\x08\n\x04NONE\x10\x00\x12\n\n\x06\x42\x41SE64\x10\x01\x42\x86\x02\nYio.envoyproxy.envoy.extensions.filters.network.thrift_proxy.filters.header_to_metadata.v3B\x15HeaderToMetadataProtoP\x01Z\x87\x01github.com/envoyproxy/go-control-plane/envoy/extensions/filters/network/thrift_proxy/filters/header_to_metadata/v3;header_to_metadatav3\xba\x80\xc8\xd1\x06\x02\x10\x02\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.extensions.filters.network.thrift_proxy.filters.header_to_metadata.v3.header_to_metadata_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\nYio.envoyproxy.envoy.extensions.filters.network.thrift_proxy.filters.header_to_metadata.v3B\025HeaderToMetadataProtoP\001Z\207\001github.com/envoyproxy/go-control-plane/envoy/extensions/filters/network/thrift_proxy/filters/header_to_metadata/v3;header_to_metadatav3\272\200\310\321\006\002\020\002'
  _globals['_HEADERTOMETADATA_KEYVALUEPAIR'].fields_by_name['key']._loaded_options = None
  _globals['_HEADERTOMETADATA_KEYVALUEPAIR'].fields_by_name['key']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_HEADERTOMETADATA_KEYVALUEPAIR'].fields_by_name['type']._loaded_options = None
  _globals['_HEADERTOMETADATA_KEYVALUEPAIR'].fields_by_name['type']._serialized_options = b'\372B\005\202\001\002\020\001'
  _globals['_HEADERTOMETADATA_RULE'].fields_by_name['header']._loaded_options = None
  _globals['_HEADERTOMETADATA_RULE'].fields_by_name['header']._serialized_options = b'\372B\nr\010\020\001\300\001\001\310\001\000'
  _globals['_HEADERTOMETADATA'].fields_by_name['request_rules']._loaded_options = None
  _globals['_HEADERTOMETADATA'].fields_by_name['request_rules']._serialized_options = b'\372B\005\222\001\002\010\001'
  _globals['_HEADERTOMETADATA']._serialized_start=273
  _globals['_HEADERTOMETADATA']._serialized_end=1260
  _globals['_HEADERTOMETADATA_KEYVALUEPAIR']._serialized_start=428
  _globals['_HEADERTOMETADATA_KEYVALUEPAIR']._serialized_end=854
  _globals['_HEADERTOMETADATA_RULE']._serialized_start=857
  _globals['_HEADERTOMETADATA_RULE']._serialized_end=1166
  _globals['_HEADERTOMETADATA_VALUETYPE']._serialized_start=1168
  _globals['_HEADERTOMETADATA_VALUETYPE']._serialized_end=1223
  _globals['_HEADERTOMETADATA_VALUEENCODE']._serialized_start=1225
  _globals['_HEADERTOMETADATA_VALUEENCODE']._serialized_end=1260
# @@protoc_insertion_point(module_scope)
